@@ -33,7 +33,7 @@ oid2arcs(OBJECT_IDENTIFIER_t *oid, struct oid_arcs *result)
 
 	count = OBJECT_IDENTIFIER_get_arcs(oid, result->arcs, MAX_ARCS);
 	if (count < 0) {
-		pr_err("OBJECT_IDENTIFIER_get_arcs() returned %zd.", count);
+		pr_val_err("OBJECT_IDENTIFIER_get_arcs() returned %zd.", count);
 		free(result->arcs);
 		return count;
 	}
@@ -51,7 +51,7 @@ oid2arcs(OBJECT_IDENTIFIER_t *oid, struct oid_arcs *result)
 
 		count2 = OBJECT_IDENTIFIER_get_arcs(oid, result->arcs, count);
 		if (count != count2) {
-			pr_err("OBJECT_IDENTIFIER_get_arcs() returned %zd. (expected %zd)",
+			pr_val_err("OBJECT_IDENTIFIER_get_arcs() returned %zd. (expected %zd)",
 			    count2, count);
 			free(result->arcs);
 			return -EINVAL;
