@@ -15,6 +15,7 @@
 #include "log.h"
 #include "config/boolean.h"
 #include "config/incidences.h"
+#include "config/rrdp_conf.h"
 #include "config/str.h"
 #include "config/sync_strategy.h"
 #include "config/uint.h"
@@ -61,7 +62,7 @@ struct rpki_config {
 
 	struct {
 		/** The bound listening address of the RTR server. */
-		char *address;
+		struct string_array address;
 		/** The bound listening port of the RTR server. */
 		char *port;
 		/** Outstanding connections in the socket's listen queue */
@@ -118,6 +119,22 @@ struct rpki_config {
 	} rrdp;
 
 	struct {
+		/* Enables the protocol */
+		bool enabled;
+		/*
+		 * Priority, whenever there's an option to sync something via
+		 * http or rsync, use this priority. When working with CAs, this
+		 * will override the order set at the CAs in their accessMethod
+		 * extension.
+		 */
+		unsigned int priority;
+		/* Retry conf, utilized on errors */
+		struct {
+			/* Maximum number of retries on error */
+			unsigned int count;
+			/* Interval (in seconds) between each retry */
+			unsigned int interval;
+		} retry;
 		/* User-Agent header set at requests */
 		char *user_agent;
 		/* Timeout in seconds for the connect phase */
@@ -131,6 +148,10 @@ struct rpki_config {
 	} http;
 
 	struct {
+		/** Enables operation logs **/
+		bool enabled;
+		/** String tag to identify operation logs **/
+		char *tag;
 		/** Print ANSI color codes? */
 		bool color;
 		/** Format in which file names will be printed. */
@@ -139,8 +160,27 @@ struct rpki_config {
 		uint8_t level;
 		/* Log output */
 		enum log_output output;
+		/** facility for syslog if output is syslog **/
+		uint32_t facility;
 	} log;
 
+	struct {
+		/** Enables validation Logs **/
+		bool enabled;
+		/** String tag to identify validation logs **/
+		char *tag;
+		/** Print ANSI color codes? */
+		bool color;
+		/** Format in which file names will be printed. */
+		enum filename_format filename_format;
+		/* Log level */
+		uint8_t level;
+		/* Log output */
+		enum log_output output;
+		/** facilities for syslog if output is syslog **/
+		uint32_t facility;
+	} validation_log;
+
 	struct {
 		/** File where the validated ROAs will be stored */
 		char *roa;
@@ -150,6 +190,9 @@ struct rpki_config {
 
 	/* ASN1 decoder max stack size allowed */
 	unsigned int asn1_decode_max_stack;
+
+	/* Time period that must lapse to warn about a stale repository */
+	unsigned int stale_repository_period;
 };
 
 static void print_usage(FILE *, bool);
@@ -274,15 +317,17 @@ static const struct option_field options[] = {
 	{
 		.id = 5000,
 		.name = "server.address",
-		.type = &gt_string,
+		.type = &gt_string_array,
 		.offset = offsetof(struct rpki_config, server.address),
-		.doc = "Address to which RTR server will bind itself to. Can be a name, in which case an address will be resolved.",
+		.doc = "List of addresses (comma separated) to which RTR server will bind itself to. Can be a name, in which case an address will be resolved. The format for each address is '<address>[#<port/service>]'.",
+		.min = 0,
+		.max = 50,
 	}, {
 		.id = 5001,
 		.name = "server.port",
 		.type = &gt_string,
 		.offset = offsetof(struct rpki_config, server.port),
-		.doc = "Port to which RTR server will bind itself to. Can be a string, in which case a number will be resolved.",
+		.doc = "Default port to which RTR server addresses will bind itself to. Can be a string, in which case a number will be resolved. If all of the addresses have a port, this value isn't utilized.",
 	}, {
 		.id = 5002,
 		.name = "server.backlog",
@@ -412,6 +457,8 @@ static const struct option_field options[] = {
 		.offset = offsetof(struct rpki_config, rsync.args.recursive),
 		.doc = "RSYNC program arguments that will trigger a recursive RSYNC",
 		.availability = AVAILABILITY_JSON,
+		/* Unlimited */
+		.max = 0,
 	}, {
 		.id = 3007,
 		.name = "rsync.arguments-flat",
@@ -419,37 +466,39 @@ static const struct option_field options[] = {
 		.offset = offsetof(struct rpki_config, rsync.args.flat),
 		.doc = "RSYNC program arguments that will trigger a non-recursive RSYNC",
 		.availability = AVAILABILITY_JSON,
+		/* Unlimited */
+		.max = 0,
 	},
 
 	/* RRDP fields */
 	{
 		.id = 10000,
 		.name = "rrdp.enabled",
-		.type = &gt_bool,
+		.type = &gt_rrdp_enabled,
 		.offset = offsetof(struct rpki_config, rrdp.enabled),
-		.doc = "Enables RRDP execution",
+		.doc = "Enables RRDP execution. Will be deprecated, use 'http.enabled' instead.",
 	}, {
 		.id = 10001,
 		.name = "rrdp.priority",
-		.type = &gt_uint32,
+		.type = &gt_rrdp_priority,
 		.offset = offsetof(struct rpki_config, rrdp.priority),
-		.doc = "Priority of execution to fetch repositories files, a higher value means higher priority",
+		.doc = "Priority of execution to fetch repositories files, a higher value means higher priority. Will be deprecated, use 'http.priority' instead.",
 		.min = 0,
 		.max = 100,
 	}, {
 		.id = 10002,
 		.name = "rrdp.retry.count",
-		.type = &gt_uint,
+		.type = &gt_rrdp_retry_count,
 		.offset = offsetof(struct rpki_config, rrdp.retry.count),
-		.doc = "Maximum amount of retries whenever there's an error fetching RRDP files",
+		.doc = "Maximum amount of retries whenever there's an error fetching RRDP files. Will be deprecated, use 'http.retry.count' instead.",
 		.min = 0,
 		.max = UINT_MAX,
 	}, {
 		.id = 10003,
 		.name = "rrdp.retry.interval",
-		.type = &gt_uint,
+		.type = &gt_rrdp_retry_interval,
 		.offset = offsetof(struct rpki_config, rrdp.retry.interval),
-		.doc = "Period (in seconds) to wait between retries after an error ocurred fetching RRDP files",
+		.doc = "Period (in seconds) to wait between retries after an error ocurred fetching RRDP files. Will be deprecated, use 'http.retry.interval' instead.",
 		.min = 0,
 		.max = UINT_MAX,
 	},
@@ -457,13 +506,47 @@ static const struct option_field options[] = {
 	/* HTTP requests parameters */
 	{
 		.id = 9000,
+		.name = "http.enabled",
+		.type = &gt_rrdp_enabled,
+		.offset = offsetof(struct rpki_config, http.enabled),
+		.doc = "Enables outgoing HTTP requests",
+	},
+	{
+		.id = 9001,
+		.name = "http.priority",
+		.type = &gt_rrdp_priority,
+		.offset = offsetof(struct rpki_config, http.priority),
+		.doc = "Priority of execution to fetch repositories files, a higher value means higher priority",
+		.min = 0,
+		.max = 100,
+	},
+	{
+		.id = 9002,
+		.name = "http.retry.count",
+		.type = &gt_rrdp_retry_count,
+		.offset = offsetof(struct rpki_config, http.retry.count),
+		.doc = "Maximum amount of retries whenever there's an error requesting HTTP URIs",
+		.min = 0,
+		.max = UINT_MAX,
+	},
+	{
+		.id = 9003,
+		.name = "http.retry.interval",
+		.type = &gt_rrdp_retry_interval,
+		.offset = offsetof(struct rpki_config, http.retry.interval),
+		.doc = "Period (in seconds) to wait between retries after an error ocurred doing HTTP requests",
+		.min = 0,
+		.max = UINT_MAX,
+	},
+	{
+		.id = 9004,
 		.name = "http.user-agent",
 		.type = &gt_string,
 		.offset = offsetof(struct rpki_config, http.user_agent),
 		.doc = "User-Agent to use at HTTP requests, eg. Fort Validator Local/1.0",
 	},
 	{
-		.id = 9001,
+		.id = 9005,
 		.name = "http.connect-timeout",
 		.type = &gt_uint,
 		.offset = offsetof(struct rpki_config, http.connect_timeout),
@@ -472,7 +555,7 @@ static const struct option_field options[] = {
 		.max = UINT_MAX,
 	},
 	{
-		.id = 9002,
+		.id = 9006,
 		.name = "http.transfer-timeout",
 		.type = &gt_uint,
 		.offset = offsetof(struct rpki_config, http.transfer_timeout),
@@ -481,7 +564,7 @@ static const struct option_field options[] = {
 		.max = UINT_MAX,
 	},
 	{
-		.id = 9003,
+		.id = 9007,
 		.name = "http.idle-timeout",
 		.type = &gt_uint,
 		.offset = offsetof(struct rpki_config, http.idle_timeout),
@@ -490,7 +573,7 @@ static const struct option_field options[] = {
 		.max = UINT_MAX,
 	},
 	{
-		.id = 9004,
+		.id = 9008,
 		.name = "http.ca-path",
 		.type = &gt_string,
 		.offset = offsetof(struct rpki_config, http.ca_path),
@@ -500,29 +583,94 @@ static const struct option_field options[] = {
 
 	/* Logging fields */
 	{
-		.id = 'c',
-		.name = "log.color-output",
-		.type = &gt_bool,
-		.offset = offsetof(struct rpki_config, log.color),
-		.doc = "Print ANSI color codes",
-	}, {
 		.id = 4000,
-		.name = "log.file-name-format",
-		.type = &gt_filename_format,
-		.offset = offsetof(struct rpki_config, log.filename_format),
-		.doc = "File name variant to print during debug/error messages",
+		.name = "log.enabled",
+		.type = &gt_bool,
+		.offset = offsetof(struct rpki_config, log.enabled),
+		.doc = "Enables operation logs",
 	}, {
 		.id = 4001,
+		.name = "log.output",
+		.type = &gt_log_output,
+		.offset = offsetof(struct rpki_config, log.output),
+		.doc = "Output where operation log messages will be printed",
+	}, {
+		.id = 4002,
 		.name = "log.level",
 		.type = &gt_log_level,
 		.offset = offsetof(struct rpki_config, log.level),
 		.doc = "Log level to print message of equal or higher importance",
 	}, {
-		.id = 4002,
-		.name = "log.output",
+		.id = 4003,
+		.name = "log.tag",
+		.type = &gt_string,
+		.offset = offsetof(struct rpki_config, log.tag),
+		.doc = "Text tag to identify operation logs",
+		.arg_doc = "<string>",
+	}, {
+		.id = 4004,
+		.name = "log.facility",
+		.type = &gt_log_facility,
+		.offset = offsetof(struct rpki_config, log.facility),
+		.doc = "Facility for syslog if output is syslog",
+	}, {
+		.id = 4005,
+		.name = "log.file-name-format",
+		.type = &gt_filename_format,
+		.offset = offsetof(struct rpki_config, log.filename_format),
+		.doc = "File name variant to print during debug/error messages",
+	}, {
+		.id = 'c',
+		.name = "log.color-output",
+		.type = &gt_bool,
+		.offset = offsetof(struct rpki_config, log.color),
+		.doc = "Print ANSI color codes",
+	},
+
+	{
+		.id = 4010,
+		.name = "validation-log.enabled",
+		.type = &gt_bool,
+		.offset = offsetof(struct rpki_config, validation_log.enabled),
+		.doc = "Enables validation logs",
+	}, {
+		.id = 4011,
+		.name = "validation-log.output",
 		.type = &gt_log_output,
-		.offset = offsetof(struct rpki_config, log.output),
-		.doc = "Output where log messages will be printed",
+		.offset = offsetof(struct rpki_config, validation_log.output),
+		.doc = "Output where validation log messages will be printed",
+	}, {
+		.id = 4012,
+		.name = "validation-log.level",
+		.type = &gt_log_level,
+		.offset = offsetof(struct rpki_config, validation_log.level),
+		.doc = "Log level to print message of equal or higher importance",
+	}, {
+		.id = 4013,
+		.name = "validation-log.tag",
+		.type = &gt_string,
+		.offset = offsetof(struct rpki_config, validation_log.tag),
+		.doc = "Text tag to identify validation logs",
+		.arg_doc = "<string>",
+	}, {
+		.id = 4014,
+		.name = "validation-log.facility",
+		.type = &gt_log_facility,
+		.offset = offsetof(struct rpki_config, validation_log.facility),
+		.doc = "Facility for syslog if output is syslog",
+	}, {
+		.id = 4015,
+		.name = "validation-log.file-name-format",
+		.type = &gt_filename_format,
+		.offset = offsetof(struct rpki_config,
+		    validation_log.filename_format),
+		.doc = "File name variant to print during debug/error messages",
+	}, {
+		.id = 4016,
+		.name = "validation-log.color-output",
+		.type = &gt_bool,
+		.offset = offsetof(struct rpki_config, validation_log.color),
+		.doc = "Print ANSI color codes",
 	},
 
 	/* Incidences */
@@ -560,6 +708,15 @@ static const struct option_field options[] = {
 		.min = 1,
 		.max = UINT_MAX,
 	},
+	{
+		.id = 8001,
+		.name = "stale-repository-period",
+		.type = &gt_uint,
+		.offset = offsetof(struct rpki_config, stale_repository_period),
+		.doc = "Time period that must lapse to warn about stale repositories",
+		.min = 0,
+		.max = UINT_MAX,
+	},
 
 	{ 0 },
 };
@@ -679,13 +836,13 @@ print_config(void)
 {
 	struct option_field const *opt;
 
-	pr_info("Configuration {");
+	pr_op_info("Configuration {");
 
 	FOREACH_OPTION(options, opt, 0xFFFF)
 		if (is_rpki_config_field(opt) && opt->type->print != NULL)
 			opt->type->print(opt, get_rpki_config_field(opt));
 
-	pr_info("}");
+	pr_op_info("}");
 }
 
 static int
@@ -717,11 +874,16 @@ set_default_values(void)
 	 * duplicates.
 	 */
 
-	rpki_config.server.address = NULL;
-	rpki_config.server.port = strdup("323");
-	if (rpki_config.server.port == NULL)
+	error = string_array_init(&rpki_config.server.address, NULL, 0);
+	if (error)
 		return pr_enomem();
 
+	rpki_config.server.port = strdup("323");
+	if (rpki_config.server.port == NULL) {
+		error = pr_enomem();
+		goto revert_address;
+	}
+
 	rpki_config.server.backlog = SOMAXCONN;
 	rpki_config.server.interval.validation = 3600;
 	rpki_config.server.interval.refresh = 3600;
@@ -764,11 +926,11 @@ set_default_values(void)
 	if (error)
 		goto revert_recursive_array;
 
-	rpki_config.rrdp.enabled = true;
-	rpki_config.rrdp.priority = 50;
-	rpki_config.rrdp.retry.count = 2;
-	rpki_config.rrdp.retry.interval = 5;
-
+	/* By default, has a higher priority than rsync */
+	rpki_config.http.enabled = true;
+	rpki_config.http.priority = 60;
+	rpki_config.http.retry.count = 2;
+	rpki_config.http.retry.interval = 5;
 	rpki_config.http.user_agent = strdup(PACKAGE_NAME "/" PACKAGE_VERSION);
 	if (rpki_config.http.user_agent == NULL) {
 		error = pr_enomem();
@@ -779,17 +941,49 @@ set_default_values(void)
 	rpki_config.http.idle_timeout = 15;
 	rpki_config.http.ca_path = NULL; /* Use system default */
 
+	/*
+	 * FIXME (later) Same values as http.*, delete when rrdp.* is fully
+	 * deprecated
+	 */
+	rpki_config.rrdp.enabled = rpki_config.http.enabled;
+	rpki_config.rrdp.priority = rpki_config.http.priority;
+	rpki_config.rrdp.retry.count = rpki_config.http.retry.count;
+	rpki_config.rrdp.retry.interval = rpki_config.http.retry.interval;
+
 	rpki_config.log.color = false;
 	rpki_config.log.filename_format = FNF_GLOBAL;
 	rpki_config.log.level = LOG_WARNING;
 	rpki_config.log.output = CONSOLE;
 
+	rpki_config.log.enabled = true;
+	rpki_config.log.output = CONSOLE;
+	rpki_config.log.level = LOG_WARNING;
+	rpki_config.log.color = false;
+	rpki_config.log.filename_format = FNF_GLOBAL;
+	rpki_config.log.facility = LOG_DAEMON;
+	rpki_config.log.tag = NULL;
+
+	rpki_config.validation_log.enabled = false;
+	rpki_config.validation_log.output = CONSOLE;
+	rpki_config.validation_log.level = LOG_WARNING;
+	rpki_config.validation_log.color = false;
+	rpki_config.validation_log.filename_format = FNF_GLOBAL;
+	rpki_config.validation_log.facility = LOG_DAEMON;
+	rpki_config.validation_log.tag = strdup("Validation");
+	if (rpki_config.validation_log.tag == NULL) {
+		error = pr_enomem();
+		goto revert_validation_log_tag;
+	}
+
 	rpki_config.output.roa = NULL;
 	rpki_config.output.bgpsec = NULL;
 
 	rpki_config.asn1_decode_max_stack = 4096; /* 4kB */
+	rpki_config.stale_repository_period = 43200; /* 12 hours */
 
 	return 0;
+revert_validation_log_tag:
+	free(rpki_config.http.user_agent);
 revert_flat_array:
 	string_array_cleanup(&rpki_config.rsync.args.flat);
 revert_recursive_array:
@@ -800,6 +994,8 @@ revert_repository:
 	free(rpki_config.local_repository);
 revert_port:
 	free(rpki_config.server.port);
+revert_address:
+	string_array_cleanup(&rpki_config.server.address);
 	return error;
 }
 
@@ -813,28 +1009,28 @@ static int
 validate_config(void)
 {
 	if (rpki_config.tal == NULL)
-		return pr_err("The TAL file/directory (--tal) is mandatory.");
+		return pr_op_err("The TAL file/directory (--tal) is mandatory.");
 
-	if (!valid_file_or_dir(rpki_config.tal, true, true))
-		return pr_err("Invalid TAL file/directory.");
+	if (!valid_file_or_dir(rpki_config.tal, true, true, pr_op_errno))
+		return pr_op_err("Invalid TAL file/directory.");
 
 	if (rpki_config.server.interval.expire <
 	    rpki_config.server.interval.refresh ||
 	    rpki_config.server.interval.expire <
 	    rpki_config.server.interval.retry)
-		return pr_err("Expire interval must be greater than refresh and retry intervals");
+		return pr_op_err("Expire interval must be greater than refresh and retry intervals");
 
 	if (rpki_config.output.roa != NULL &&
 	    !valid_output_file(rpki_config.output.roa))
-		return pr_err("Invalid output.roa file.");
+		return pr_op_err("Invalid output.roa file.");
 
 	if (rpki_config.output.bgpsec != NULL &&
 	    !valid_output_file(rpki_config.output.bgpsec))
-		return pr_err("Invalid output.bgpsec file.");
+		return pr_op_err("Invalid output.bgpsec file.");
 
 	if (rpki_config.slurm != NULL &&
-	    !valid_file_or_dir(rpki_config.slurm, true, true))
-		return pr_err("Invalid slurm location.");
+	    !valid_file_or_dir(rpki_config.slurm, true, true, pr_op_errno))
+		return pr_op_err("Invalid slurm location.");
 
 	/* FIXME (later) Remove when sync-strategy is fully deprecated */
 	if (!rpki_config.rsync.enabled)
@@ -893,7 +1089,7 @@ handle_opt(int opt)
 		}
 	}
 
-	pr_err("Unrecognized option: %d", opt);
+	pr_op_err("Unrecognized option: %d", opt);
 	return -ESRCH;
 }
 
@@ -930,7 +1126,7 @@ handle_flags_config(int argc, char **argv)
 	 * This program does not have unflagged payload.
 	 */
 	if (optind < argc) {
-		error = pr_err("I don't know what '%s' is.", argv[optind]);
+		error = pr_op_err("I don't know what '%s' is.", argv[optind]);
 		goto end;
 	}
 
@@ -938,9 +1134,11 @@ handle_flags_config(int argc, char **argv)
 
 	log_start();
 end:
-	if (error)
+	if (error) {
 		free_rpki_config();
-	else
+		pr_op_err("Try '%s --usage' or '%s --help' for more information.",
+		    program_name, program_name);
+	} else
 		print_config();
 
 	free(long_opts);
@@ -961,10 +1159,10 @@ config_get_mode(void)
 	return rpki_config.mode;
 }
 
-char const *
+struct string_array const *
 config_get_server_address(void)
 {
-	return rpki_config.server.address;
+	return &rpki_config.server.address;
 }
 
 char const *
@@ -1041,29 +1239,89 @@ config_get_max_cert_depth(void)
 }
 
 bool
-config_get_color_output(void)
+config_get_op_log_enabled(void)
+{
+	return rpki_config.log.enabled;
+}
+
+char const *
+config_get_op_log_tag(void)
+{
+	return rpki_config.log.tag;
+}
+
+bool
+config_get_op_log_color_output(void)
 {
 	return rpki_config.log.color;
 }
 
 enum filename_format
-config_get_filename_format(void)
+config_get_op_log_filename_format(void)
 {
 	return rpki_config.log.filename_format;
 }
 
 uint8_t
-config_get_log_level(void)
+config_get_op_log_level(void)
 {
 	return rpki_config.log.level;
 }
 
 enum log_output
-config_get_log_output(void)
+config_get_op_log_output(void)
 {
 	return rpki_config.log.output;
 }
 
+uint32_t
+config_get_op_log_facility(void)
+{
+	return rpki_config.log.facility;
+}
+
+bool
+config_get_val_log_enabled(void)
+{
+	return rpki_config.validation_log.enabled;
+}
+
+char const *
+config_get_val_log_tag(void)
+{
+	return rpki_config.validation_log.tag;
+}
+
+bool
+config_get_val_log_color_output(void)
+{
+	return rpki_config.validation_log.color;
+}
+
+enum filename_format
+config_get_val_log_filename_format(void)
+{
+	return rpki_config.validation_log.filename_format;
+}
+
+uint8_t
+config_get_val_log_level(void)
+{
+	return rpki_config.validation_log.level;
+}
+
+enum log_output
+config_get_val_log_output(void)
+{
+	return rpki_config.validation_log.output;
+}
+
+uint32_t
+config_get_val_log_facility(void)
+{
+	return rpki_config.validation_log.facility;
+}
+
 bool
 config_get_rsync_enabled(void)
 {
@@ -1120,27 +1378,27 @@ config_get_rsync_args(bool is_ta)
 }
 
 bool
-config_get_rrdp_enabled(void)
+config_get_http_enabled(void)
 {
-	return rpki_config.rrdp.enabled;
+	return rpki_config.http.enabled;
 }
 
 unsigned int
-config_get_rrdp_priority(void)
+config_get_http_priority(void)
 {
-	return rpki_config.rrdp.priority;
+	return rpki_config.http.priority;
 }
 
 unsigned int
-config_get_rrdp_retry_count(void)
+config_get_http_retry_count(void)
 {
-	return rpki_config.rrdp.retry.count;
+	return rpki_config.http.retry.count;
 }
 
 unsigned int
-config_get_rrdp_retry_interval(void)
+config_get_http_retry_interval(void)
 {
-	return rpki_config.rrdp.retry.interval;
+	return rpki_config.http.retry.interval;
 }
 
 char const *
@@ -1191,12 +1449,37 @@ config_get_asn1_decode_max_stack(void)
 	return rpki_config.asn1_decode_max_stack;
 }
 
+unsigned int
+config_get_stale_repository_period(void)
+{
+	return rpki_config.stale_repository_period;
+}
+
 void
 config_set_rsync_enabled(bool value)
 {
 	rpki_config.rsync.enabled = value;
 }
 
+void
+config_set_http_enabled(bool value)
+{
+	rpki_config.http.enabled = value;
+}
+
+void
+free_rpki_config(void)
+{
+	struct option_field const *option;
+
+	FOREACH_OPTION(options, option, 0xFFFF)
+		if (is_rpki_config_field(option) && option->type->free != NULL)
+			option->type->free(get_rpki_config_field(option));
+}
+
+/*
+ * "To be deprecated" section
+ */
 void
 config_set_rrdp_enabled(bool value)
 {
@@ -1216,11 +1499,37 @@ config_set_rsync_strategy(enum rsync_strategy value)
 }
 
 void
-free_rpki_config(void)
+config_set_rrdp_priority(unsigned int value)
 {
-	struct option_field const *option;
+	rpki_config.rrdp.priority = value;
+}
 
-	FOREACH_OPTION(options, option, 0xFFFF)
-		if (is_rpki_config_field(option) && option->type->free != NULL)
-			option->type->free(get_rpki_config_field(option));
+void
+config_set_http_priority(unsigned int value)
+{
+	rpki_config.http.priority = value;
+}
+
+void
+config_set_rrdp_retry_count(unsigned int value)
+{
+	rpki_config.rrdp.retry.count = value;
+}
+
+void
+config_set_http_retry_count(unsigned int value)
+{
+	rpki_config.http.retry.count = value;
+}
+
+void
+config_set_rrdp_retry_interval(unsigned int value)
+{
+	rpki_config.rrdp.retry.interval = value;
+}
+
+void
+config_set_http_retry_interval(unsigned int value)
+{
+	rpki_config.http.retry.interval = value;
 }
