@@ -5,6 +5,7 @@
 #include <sys/stat.h>
 #include <sys/types.h> /* For blksize_t */
 
+#include "common.h"
 #include "file.h"
 #include "log.h"
 #include "asn1/oid.h"
@@ -16,7 +17,7 @@ get_md(char const *algorithm, EVP_MD const **result)
 
 	md = EVP_get_digestbyname(algorithm);
 	if (md == NULL) {
-		pr_err("Unknown message digest %s", algorithm);
+		pr_val_err("Unknown message digest %s", algorithm);
 		return -EINVAL;
 	}
 
@@ -35,6 +36,14 @@ hash_matches(unsigned char const *expected, size_t expected_len,
 static int
 hash_file(char const *algorithm, struct rpki_uri *uri, unsigned char *result,
     unsigned int *result_len)
+{
+	return hash_local_file(algorithm, uri_get_local(uri), result,
+		result_len);
+}
+
+int
+hash_local_file(char const *algorithm, char const *uri, unsigned char *result,
+    unsigned int *result_len)
 {
 	EVP_MD const *md;
 	FILE *file;
@@ -49,7 +58,7 @@ hash_file(char const *algorithm, struct rpki_uri *uri, unsigned char *result,
 	if (error)
 		return error;
 
-	error = file_open(uri_get_local(uri), &file, &stat);
+	error = file_open(uri, &file, &stat);
 	if (error)
 		return error;
 
@@ -67,7 +76,7 @@ hash_file(char const *algorithm, struct rpki_uri *uri, unsigned char *result,
 	}
 
 	if (!EVP_DigestInit_ex(ctx, md, NULL)) {
-		error = crypto_err("EVP_DigestInit_ex() failed");
+		error = val_crypto_err("EVP_DigestInit_ex() failed");
 		goto end3;
 	}
 
@@ -75,20 +84,20 @@ hash_file(char const *algorithm, struct rpki_uri *uri, unsigned char *result,
 		consumed = fread(buffer, 1, buffer_len, file);
 		error = ferror(file);
 		if (error) {
-			pr_errno(error,
+			pr_val_errno(error,
 			    "File reading error. Error message (apparently)");
 			goto end3;
 		}
 
 		if (!EVP_DigestUpdate(ctx, buffer, consumed)) {
-			error = crypto_err("EVP_DigestUpdate() failed");
+			error = val_crypto_err("EVP_DigestUpdate() failed");
 			goto end3;
 		}
 
 	} while (!feof(file));
 
 	if (!EVP_DigestFinal_ex(ctx, result, result_len))
-		error = crypto_err("EVP_DigestFinal_ex() failed");
+		error = val_crypto_err("EVP_DigestFinal_ex() failed");
 
 end3:
 	EVP_MD_CTX_free(ctx);
@@ -101,7 +110,14 @@ end1:
 
 /**
  * Computes the hash of the file @uri, and compares it to @expected (The
- * "expected" hash). Returns 0 if no errors happened and the hashes match.
+ * "expected" hash).
+ *
+ * Returns:
+ *   0 if no errors happened and the hashes match, or the hash doesn't match
+ *     but there's an incidence to ignore such error.
+ * < 0 if there was an error that can't be ignored.
+ * > 0 if there was an error but it can be ignored (file not found and there's
+ *     an incidence to ignore this).
  */
 int
 hash_validate_mft_file(char const *algorithm, struct rpki_uri *uri,
@@ -112,15 +128,29 @@ hash_validate_mft_file(char const *algorithm, struct rpki_uri *uri,
 	int error;
 
 	if (expected->bits_unused != 0)
-		return pr_err("Hash string has unused bits.");
+		return pr_val_err("Hash string has unused bits.");
 
-	error = hash_file(algorithm, uri, actual, &actual_len);
-	if (error)
-		return error;
+	do {
+		error = hash_file(algorithm, uri, actual, &actual_len);
+		if (!error)
+			break;
+
+		if (error == EACCES || error == ENOENT) {
+			if (incidence(INID_MFT_FILE_NOT_FOUND,
+			    "File '%s' listed at manifest doesn't exist.",
+			    uri_val_get_printable(uri)))
+				return -EINVAL;
+
+			return error;
+		}
+		/* Any other error (crypto, enomem, file read) */
+		return ENSURE_NEGATIVE(error);
+	} while (0);
 
 	if (!hash_matches(expected->buf, expected->size, actual, actual_len)) {
-		return pr_err("File '%s' does not match its manifest hash.",
-		    uri_get_printable(uri));
+		return incidence(INID_MFT_FILE_HASH_NOT_MATCH,
+		    "File '%s' does not match its manifest hash.",
+		    uri_val_get_printable(uri));
 	}
 
 	return 0;
@@ -143,8 +173,8 @@ hash_validate_file(char const *algorithm, struct rpki_uri *uri,
 		return error;
 
 	if (!hash_matches(expected, expected_len, actual, actual_len)) {
-		return pr_err("File '%s' does not match its expected hash.",
-		    uri_get_printable(uri));
+		return pr_val_err("File '%s' does not match its expected hash.",
+		    uri_val_get_printable(uri));
 	}
 
 	return 0;
@@ -170,7 +200,7 @@ hash_buffer(char const *algorithm,
 	if (!EVP_DigestInit_ex(ctx, md, NULL)
 	    || !EVP_DigestUpdate(ctx, content, content_len)
 	    || !EVP_DigestFinal_ex(ctx, hash, hash_len)) {
-		error = crypto_err("Buffer hashing failed");
+		error = val_crypto_err("Buffer hashing failed");
 	}
 
 	EVP_MD_CTX_free(ctx);
