@@ -18,6 +18,7 @@
 #include "line_file.h"
 #include "log.h"
 #include "random.h"
+#include "reqs_errors.h"
 #include "state.h"
 #include "thread_var.h"
 #include "validation_handler.h"
@@ -29,11 +30,14 @@
 #include "rrdp/db/db_rrdp.h"
 
 #define TAL_FILE_EXTENSION	".tal"
+typedef int (*foreach_uri_cb)(struct tal *, struct rpki_uri *, void *);
 
 struct uris {
 	struct rpki_uri **array; /* This is an array of rpki URIs. */
 	unsigned int count;
 	unsigned int size;
+	unsigned int rsync_count;
+	unsigned int https_count;
 };
 
 struct tal {
@@ -43,29 +47,43 @@ struct tal {
 	size_t spki_len;
 };
 
-struct fv_param {
-	int *exit_status; /* Return status of the file validation */
-	char *tal_file;
-	void *arg;
-};
-
-struct thread {
+struct validation_thread {
+	/*
+	 * pid is not guaranteed to be defined in the thread. It should only
+	 * be manipulated by the parent thread.
+	 */
 	pthread_t pid;
-	char *file;
-	int *exit_status;
-	SLIST_ENTRY(thread) next;
+	char *tal_file;
+	/*
+	 * Try to use the TA from the local cache? Only if none of the URIs
+	 * was sync'd.
+	 */
+	bool retry_local;
+	/* Try to sync the current TA URI? */
+	bool sync_files;
+	void *arg;
+	int exit_status;
+	/* This should also only be manipulated by the parent thread. */
+	SLIST_ENTRY(validation_thread) next;
 };
 
 /* List of threads, one per TAL file */
-SLIST_HEAD(threads_list, thread) threads;
+SLIST_HEAD(threads_list, validation_thread);
+
+struct tal_param {
+	struct db_table *db;
+	struct threads_list *threads;
+};
 
 static int
 uris_init(struct uris *uris)
 {
 	uris->count = 0;
+	uris->rsync_count = 0;
+	uris->https_count = 0;
 	uris->size = 4; /* Most TALs only define one. */
 	uris->array = malloc(uris->size * sizeof(struct rpki_uri *));
-	return (uris->array != NULL) ? 0 : -ENOMEM;
+	return (uris->array != NULL) ? 0 : pr_enomem();
 }
 
 static void
@@ -86,11 +104,16 @@ uris_add(struct uris *uris, char *uri)
 
 	error = uri_create_mixed_str(&new, uri, strlen(uri));
 	if (error == ENOTSUPPORTED)
-		return pr_err("TAL has non-RSYNC/HTTPS URI.");
+		return pr_op_err("TAL has non-RSYNC/HTTPS URI.");
 
 	if (error)
 		return error;
 
+	if (uri_is_rsync(new))
+		uris->rsync_count++;
+	else
+		uris->https_count++;
+
 	if (uris->count + 1 >= uris->size) {
 		uris->size *= 2;
 		tmp = realloc(uris->array,
@@ -117,10 +140,10 @@ read_uris(struct line_file *lfile, struct uris *uris)
 		return error;
 
 	if (uri == NULL)
-		return pr_err("TAL file is empty.");
+		return pr_op_err("TAL file is empty.");
 	if (strcmp(uri, "") == 0) {
 		free(uri);
-		return pr_err("There's no URI in the first line of the TAL.");
+		return pr_op_err("There's no URI in the first line of the TAL.");
 	} else if (strncmp(uri, "#", 1) == 0) {
 		/* More comments expected, or an URI */
 		do {
@@ -129,10 +152,10 @@ read_uris(struct line_file *lfile, struct uris *uris)
 			if (error)
 				return error;
 			if (uri == NULL)
-				return pr_err("TAL file ended prematurely. (Expected more comments or an URI list.)");
+				return pr_op_err("TAL file ended prematurely. (Expected more comments or an URI list.)");
 			if (strcmp(uri, "") == 0) {
 				free(uri);
-				return pr_err("TAL file comments syntax error. (Expected more comments or an URI list.)");
+				return pr_op_err("TAL file comments syntax error. (Expected more comments or an URI list.)");
 			}
 			/* Not a comment, probably the URI(s) */
 			if (strncmp(uri, "#", 1) != 0)
@@ -151,7 +174,7 @@ read_uris(struct line_file *lfile, struct uris *uris)
 			return error;
 
 		if (uri == NULL)
-			return pr_err("TAL file ended prematurely. (Expected URI list, blank line and public key.)");
+			return pr_op_err("TAL file ended prematurely. (Expected URI list, blank line and public key.)");
 		if (strcmp(uri, "") == 0) {
 			free(uri);
 			return 0; /* Happy path */
@@ -235,7 +258,7 @@ base64_sanitize(struct line_file *lfile, char **out)
 			 * code. It literally doesn't say how to get an error
 			 * code.
 			 */
-			pr_errno(error,
+			pr_op_errno(error,
 			    "File reading error. Error message (apparently)");
 			goto free_result;
 		}
@@ -308,7 +331,7 @@ read_spki(struct line_file *lfile, struct tal *tal)
 	if (encoded == NULL) {
 		free(tal->spki);
 		free(tmp);
-		return crypto_err("BIO_new_mem_buf() returned NULL");
+		return op_crypto_err("BIO_new_mem_buf() returned NULL");
 	}
 
 	error = base64_decode(encoded, tal->spki, true, alloc_size,
@@ -333,13 +356,13 @@ tal_load(char const *file_name, struct tal **result)
 
 	error = lfile_open(file_name, &lfile);
 	if (error) {
-		pr_errno(error, "Error opening file '%s'", file_name);
+		pr_op_errno(error, "Error opening file '%s'", file_name);
 		goto fail4;
 	}
 
 	tal = malloc(sizeof(struct tal));
 	if (tal == NULL) {
-		error = -ENOMEM;
+		error = pr_enomem();
 		goto fail3;
 	}
 
@@ -396,7 +419,7 @@ foreach_uri(struct tal *tal, foreach_uri_cb cb, void *arg)
 	return 0;
 }
 
-void
+static void
 tal_shuffle_uris(struct tal *tal)
 {
 	struct rpki_uri **array = tal->uris.array;
@@ -415,6 +438,48 @@ tal_shuffle_uris(struct tal *tal)
 	}
 }
 
+static int
+tal_order_uris(struct tal *tal)
+{
+	struct rpki_uri **ordered;
+	struct rpki_uri **tmp;
+	bool http_first;
+	unsigned int i;
+	unsigned int last_rsync;
+	unsigned int last_https;
+
+	/* First do the shuffle */
+	if (config_get_shuffle_tal_uris())
+		tal_shuffle_uris(tal);
+
+	if (config_get_rsync_priority() == config_get_http_priority())
+		return 0;
+
+	/* Now order according to the priority */
+	http_first = (config_get_http_priority() > config_get_rsync_priority());
+
+	ordered = malloc(tal->uris.size * sizeof(struct rpki_uri *));
+	if (ordered == NULL)
+		return pr_enomem();
+
+	last_rsync = (http_first ? tal->uris.https_count : 0);
+	last_https = (http_first ? 0 : tal->uris.rsync_count);
+
+	for (i = 0; i < tal->uris.count; i++) {
+		if (uri_is_rsync(tal->uris.array[i]))
+			ordered[last_rsync++] = tal->uris.array[i];
+		else
+			ordered[last_https++] = tal->uris.array[i];
+	}
+
+	/* Everything is ok, point to the ordered array */
+	tmp = tal->uris.array;
+	tal->uris.array = ordered;
+	free(tmp);
+
+	return 0;
+}
+
 char const *
 tal_get_file_name(struct tal *tal)
 {
@@ -445,7 +510,8 @@ write_http_cer(unsigned char *content, size_t size, size_t nmemb, void *arg)
 static int
 handle_https_uri(struct rpki_uri *uri)
 {
-	return http_download_file(uri, write_http_cer);
+	return http_download_file(uri, write_http_cer,
+	    reqs_errors_log_uri(uri_get_global(uri)));
 }
 
 /**
@@ -469,6 +535,7 @@ handle_tal_uri(struct tal *tal, struct rpki_uri *uri, void *arg)
 	 * A "hard error" is any other error.
 	 */
 
+	struct validation_thread *thread_arg = arg;
 	struct validation_handler validation_handler;
 	struct validation *state;
 	struct cert_stack *certstack;
@@ -478,28 +545,57 @@ handle_tal_uri(struct tal *tal, struct rpki_uri *uri, void *arg)
 	validation_handler.handle_roa_v4 = handle_roa_v4;
 	validation_handler.handle_roa_v6 = handle_roa_v6;
 	validation_handler.handle_router_key = handle_router_key;
-	validation_handler.arg = arg;
+	validation_handler.arg = thread_arg->arg;
 
 	error = validation_prepare(&state, tal, &validation_handler);
 	if (error)
 		return ENSURE_NEGATIVE(error);
 
-	if (uri_is_rsync(uri))
-		error = download_files(uri, true, false);
-	else
+	do {
+		if (!thread_arg->sync_files) {
+			/* Look for local files */
+			if (!valid_file_or_dir(uri_get_local(uri), true, false,
+			    pr_val_errno)) {
+				validation_destroy(state);
+				return 0; /* Error already logged */
+			}
+			break;
+		}
+		/* Trying to sync, considering that the sync can be disabled */
+		if (uri_is_rsync(uri)) {
+			if (!config_get_rsync_enabled()) {
+				validation_destroy(state);
+				return 0; /* Soft error */
+			}
+			error = download_files(uri, true, false);
+			break;
+		}
+		if (!config_get_http_enabled()) {
+			validation_destroy(state);
+			return 0; /* Soft error */
+		}
 		error = handle_https_uri(uri);
+	} while (0);
 
+	/* Friendly reminder: there's a positive error - EREQFAILED */
 	if (error) {
+		working_repo_push(uri_get_global(uri));
 		validation_destroy(state);
-		return pr_warn("TAL '%s' could not be downloaded.",
-		    uri_get_printable(uri));;
+		return pr_val_warn("TAL URI '%s' could not be downloaded.",
+		    uri_val_get_printable(uri));
 	}
 
-	pr_debug("TAL URI '%s' {", uri_get_printable(uri));
+	/* At least one URI was sync'd */
+	thread_arg->retry_local = false;
+	if (thread_arg->sync_files && working_repo_peek() != NULL)
+		reqs_errors_rem_uri(working_repo_peek());
+	working_repo_pop();
+
+	pr_val_debug("TAL URI '%s' {", uri_val_get_printable(uri));
 
 	if (!uri_is_certificate(uri)) {
-		error = pr_err("TAL file does not point to a certificate. (Expected .cer, got '%s')",
-		    uri_get_printable(uri));
+		error = pr_op_err("TAL URI does not point to a certificate. (Expected .cer, got '%s')",
+		    uri_op_get_printable(uri));
 		goto fail;
 	}
 
@@ -559,50 +655,85 @@ handle_tal_uri(struct tal *tal, struct rpki_uri *uri, void *arg)
 
 fail:	error = ENSURE_NEGATIVE(error);
 end:	validation_destroy(state);
-	pr_debug("}");
+	pr_val_debug("}");
 	return error;
 }
 
+static int
+__handle_tal_uri_sync(struct tal *tal, struct rpki_uri *uri, void *arg)
+{
+	int error;
+
+	error = handle_tal_uri(tal, uri, arg);
+	if (error)
+		return error;
+	working_repo_push(uri_get_global(uri));
+
+	return 0;
+}
+
+static int
+__handle_tal_uri_local(struct tal *tal, struct rpki_uri *uri, void *arg)
+{
+	return handle_tal_uri(tal, uri, arg);
+}
+
 static void *
 do_file_validation(void *thread_arg)
 {
-	struct fv_param param;
+	struct validation_thread *thread = thread_arg;
 	struct tal *tal;
 	int error;
 
-	memcpy(&param, thread_arg, sizeof(param));
-	free(thread_arg);
-
 	fnstack_init();
-	fnstack_push(param.tal_file);
+	fnstack_push(thread->tal_file);
 
-	error = tal_load(param.tal_file, &tal);
+	working_repo_init();
+
+	error = tal_load(thread->tal_file, &tal);
 	if (error)
 		goto end;
 
-	if (config_get_shuffle_tal_uris())
-		tal_shuffle_uris(tal);
-	error = foreach_uri(tal, handle_tal_uri, param.arg);
+	error = tal_order_uris(tal);
+	if (error)
+		goto destroy_tal;
+
+	error = foreach_uri(tal, __handle_tal_uri_sync, thread_arg);
+	if (error > 0) {
+		error = 0;
+		goto destroy_tal;
+	} else if (error < 0)
+		goto destroy_tal;
+
+	if (!thread->retry_local) {
+		error = pr_op_err("None of the URIs of the TAL '%s' yielded a successful traversal.",
+		    thread->tal_file);
+		goto destroy_tal;
+	}
+
+	thread->sync_files = false;
+	pr_val_warn("Looking for the TA certificate at the local files.");
+
+	error = foreach_uri(tal, __handle_tal_uri_local, thread_arg);
 	if (error > 0)
 		error = 0;
 	else if (error == 0)
-		error = pr_err("None of the URIs of the TAL '%s' yielded a successful traversal.",
-		    param.tal_file);
+		error = pr_op_err("None of the URIs of the TAL '%s' yielded a successful traversal.",
+		    thread->tal_file);
 
+destroy_tal:
 	tal_destroy(tal);
 end:
+	working_repo_cleanup();
 	fnstack_cleanup();
-	free(param.tal_file);
-	/* param.exit_error isn't released since it's from parent thread */
-	*param.exit_status = error;
+	thread->exit_status = error;
 	return NULL;
 }
 
 static void
-thread_destroy(struct thread *thread)
+thread_destroy(struct validation_thread *thread)
 {
-	free(thread->file);
-	free(thread->exit_status);
+	free(thread->tal_file);
 	free(thread);
 }
 
@@ -610,57 +741,44 @@ thread_destroy(struct thread *thread)
 static int
 __do_file_validation(char const *tal_file, void *arg)
 {
-	struct thread *thread;
-	struct fv_param *param;
-	static pthread_t pid;
-	int *exit_status;
+	struct tal_param *t_param = arg;
+	struct validation_thread *thread;
 	int error;
 
 	error = db_rrdp_add_tal(tal_file);
 	if (error)
 		return error;
 
-	exit_status = malloc(sizeof(int));
-	if (exit_status == NULL) {
+	thread = malloc(sizeof(struct validation_thread));
+	if (thread == NULL) {
 		error = pr_enomem();
 		goto free_db_rrdp;
 	}
 
-	param = malloc(sizeof(struct fv_param));
-	if (param == NULL) {
+	thread->tal_file = strdup(tal_file);
+	if (thread->tal_file == NULL) {
 		error = pr_enomem();
-		goto free_status;
+		goto free_thread;
 	}
+	thread->arg = t_param->db;
+	thread->exit_status = -EINTR;
+	thread->retry_local = true;
+	thread->sync_files = true;
 
-	param->exit_status = exit_status;
-	param->tal_file = strdup(tal_file);
-	param->arg = arg;
-
-	errno = pthread_create(&pid, NULL, do_file_validation, param);
+	errno = pthread_create(&thread->pid, NULL, do_file_validation, thread);
 	if (errno) {
-		error = -pr_errno(errno,
+		error = -pr_op_errno(errno,
 		    "Could not spawn the file validation thread");
-		goto free_param;
+		goto free_tal_file;
 	}
 
-	thread = malloc(sizeof(struct thread));
-	if (thread == NULL) {
-		close_thread(pid, tal_file);
-		error = pr_enomem();
-		goto free_param;
-	}
-
-	thread->pid = pid;
-	thread->file = strdup(tal_file);
-	thread->exit_status = exit_status;
-	SLIST_INSERT_HEAD(&threads, thread, next);
-
+	SLIST_INSERT_HEAD(t_param->threads, thread, next);
 	return 0;
-free_param:
-	free(param->tal_file);
-	free(param);
-free_status:
-	free(exit_status);
+
+free_tal_file:
+	free(thread->tal_file);
+free_thread:
+	free(thread);
 free_db_rrdp:
 	db_rrdp_rem_tal(tal_file);
 	return error;
@@ -669,17 +787,36 @@ free_db_rrdp:
 int
 perform_standalone_validation(struct db_table *table)
 {
-	struct thread *thread;
+	struct tal_param *param;
+	struct threads_list threads;
+	struct validation_thread *thread;
 	int error, t_error;
 
+	param = malloc(sizeof(struct tal_param));
+	if (param == NULL)
+		return pr_enomem();
+
 	/* Set existent tal RRDP info to non visited */
 	db_rrdp_reset_visited_tals();
 
 	SLIST_INIT(&threads);
-	error = process_file_or_dir(config_get_tal(), TAL_FILE_EXTENSION,
-	    __do_file_validation, table);
-	if (error)
+
+	param->db = table;
+	param->threads = &threads;
+
+	error = process_file_or_dir(config_get_tal(), TAL_FILE_EXTENSION, true,
+	    __do_file_validation, param);
+	if (error) {
+		/* End all threads */
+		while (!SLIST_EMPTY(&threads)) {
+			thread = threads.slh_first;
+			close_thread(thread->pid, thread->tal_file);
+			SLIST_REMOVE_HEAD(&threads, next);
+			thread_destroy(thread);
+		}
+		free(param);
 		return error;
+	}
 
 	/* Wait for all */
 	t_error = 0;
@@ -688,16 +825,22 @@ perform_standalone_validation(struct db_table *table)
 		error = pthread_join(thread->pid, NULL);
 		if (error)
 			pr_crit("pthread_join() threw %d on the '%s' thread.",
-			    error, thread->file);
+			    error, thread->tal_file);
 		SLIST_REMOVE_HEAD(&threads, next);
-		if (*thread->exit_status) {
-			t_error = *thread->exit_status;
-			pr_warn("Validation from TAL '%s' yielded error, discarding any other validation results.",
-			    thread->file);
+		if (thread->exit_status) {
+			t_error = thread->exit_status;
+			pr_op_warn("Validation from TAL '%s' yielded error, discarding any other validation results.",
+			    thread->tal_file);
 		}
 		thread_destroy(thread);
 	}
 
+	/* The parameter isn't needed anymore */
+	free(param);
+
+	/* Log the error'd URIs summary */
+	reqs_errors_log_summary();
+
 	/* One thread has errors, validation can't keep the resulting table */
 	if (t_error)
 		return t_error;
@@ -707,17 +850,3 @@ perform_standalone_validation(struct db_table *table)
 
 	return error;
 }
-
-void
-terminate_standalone_validation(void)
-{
-	struct thread *thread;
-
-	/* End all threads */
-	while (!SLIST_EMPTY(&threads)) {
-		thread = threads.slh_first;
-		close_thread(thread->pid, thread->file);
-		SLIST_REMOVE_HEAD(&threads, next);
-		thread_destroy(thread);
-	}
-}
