@@ -5,7 +5,6 @@
 #include <sys/stat.h>
 #include <ctype.h>
 #include <errno.h>
-#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -75,6 +74,7 @@ struct rdr_delta_ctx {
 struct proc_upd_args {
 	struct update_notification *parent;
 	struct visited_uris *visited_uris;
+	bool log_operation;
 };
 
 static int
@@ -110,35 +110,27 @@ write_local(unsigned char *content, size_t size, size_t nmemb, void *arg)
 }
 
 static int
-download_file(struct rpki_uri *uri, long last_update)
+download_file(struct rpki_uri *uri, long last_update, bool log_operation)
 {
-	unsigned int retries;
 	int error;
 
-	retries = 0;
-	do {
-		if (last_update > 0)
-			error = http_download_file_with_ims(uri, write_local,
-			    last_update);
-		else
-			error = http_download_file(uri, write_local);
+	if (last_update > 0)
+		error = http_download_file_with_ims(uri, write_local,
+		    last_update, log_operation);
+	else
+		error = http_download_file(uri, write_local,
+		    log_operation);
 
-		/* Remember: positive values are expected */
-		if (error >= 0)
-			return error;
+	/*
+	 * Since distinct files can be downloaded (notification, snapshot,
+	 * delta) just return the error and let the caller to add only the
+	 * update notification URI to the request errors DB.
+	 */
+	if (error == -EREQFAILED)
+		return EREQFAILED;
 
-		if (retries == config_get_rrdp_retry_count()) {
-			pr_info("Max RRDP retries (%u) reached fetching '%s', won't retry again.",
-			    retries, uri_get_global(uri));
-			return error;
-		}
-		pr_info("Retrying RRDP file download '%s' in %u seconds, %u attempts remaining.",
-		    uri_get_global(uri),
-		    config_get_rrdp_retry_interval(),
-		    config_get_rrdp_retry_count() - retries);
-		retries++;
-		sleep(config_get_rrdp_retry_interval());
-	} while (true);
+	/* Remember: positive values are expected */
+	return error;
 }
 
 /* Left trim @from, setting the result at @result pointer */
@@ -155,7 +147,7 @@ ltrim(char *from, char **result, size_t *result_size)
 		tmp_size--;
 	}
 	if (*start == '\0')
-		return pr_err("Invalid base64 encoded string (seems to be empty or full of spaces).");
+		return pr_val_err("Invalid base64 encoded string (seems to be empty or full of spaces).");
 
 	*result = start;
 	*result_size = tmp_size;
@@ -245,7 +237,7 @@ base64_read(char *content, unsigned char **out, size_t *out_len)
 
 	encoded = BIO_new_mem_buf(sanitized, -1);
 	if (encoded == NULL) {
-		error = crypto_err("BIO_new_mem_buf() returned NULL");
+		error = val_crypto_err("BIO_new_mem_buf() returned NULL");
 		goto release_sanitized;
 	}
 
@@ -287,7 +279,7 @@ parse_string(xmlTextReaderPtr reader, char const *attr, char **result)
 		xml_value = xmlTextReaderGetAttribute(reader, BAD_CAST attr);
 
 	if (xml_value == NULL)
-		return pr_err("RRDP file: Couldn't find %s from '%s'",
+		return pr_val_err("RRDP file: Couldn't find %s from '%s'",
 		    (attr == NULL ? "string content" : "xml attribute"),
 		    xmlTextReaderConstLocalName(reader));
 
@@ -313,14 +305,14 @@ parse_long(xmlTextReaderPtr reader, char const *attr, unsigned long *result)
 
 	xml_value = xmlTextReaderGetAttribute(reader, BAD_CAST attr);
 	if (xml_value == NULL)
-		return pr_err("RRDP file: Couldn't find xml attribute '%s'",
+		return pr_val_err("RRDP file: Couldn't find xml attribute '%s'",
 		    attr);
 
 	errno = 0;
 	tmp = strtoul((char *) xml_value, NULL, 10);
 	if (errno) {
 		xmlFree(xml_value);
-		pr_errno(errno, "RRDP file: Invalid long value '%s'",
+		pr_val_errno(errno, "RRDP file: Invalid long value '%s'",
 		    xml_value);
 		return -EINVAL;
 	}
@@ -343,13 +335,13 @@ parse_hex_string(xmlTextReaderPtr reader, bool required, char const *attr,
 	xml_value = xmlTextReaderGetAttribute(reader, BAD_CAST attr);
 	if (xml_value == NULL)
 		return required ?
-		    pr_err("RRDP file: Couldn't find xml attribute '%s'", attr)
+		    pr_val_err("RRDP file: Couldn't find xml attribute '%s'", attr)
 		    : 0;
 
 	/* The rest of the checks are done at the schema */
 	if (xmlStrlen(xml_value) % 2 != 0) {
 		xmlFree(xml_value);
-		return pr_err("RRDP file: Attribute %s isn't a valid hex string",
+		return pr_val_err("RRDP file: Attribute %s isn't a valid hex string",
 		    attr);
 	}
 
@@ -387,7 +379,7 @@ validate_version(xmlTextReaderPtr reader, unsigned long expected)
 		return error;
 
 	if (version != expected)
-		return pr_err("Invalid version, must be '%lu' and is '%lu'.",
+		return pr_val_err("Invalid version, must be '%lu' and is '%lu'.",
 		    expected, version);
 
 	return 0;
@@ -409,7 +401,7 @@ parse_global_data(xmlTextReaderPtr reader, struct global_data *gdata,
 	 */
 	if (!xmlStrEqual(xmlTextReaderConstNamespaceUri(reader),
 	    BAD_CAST RRDP_NAMESPACE))
-		return pr_err("Namespace isn't '%s', current value is '%s'",
+		return pr_val_err("Namespace isn't '%s', current value is '%s'",
 		    RRDP_NAMESPACE, xmlTextReaderConstNamespaceUri(reader));
 
 	error = validate_version(reader, 1);
@@ -435,7 +427,7 @@ parse_global_data(xmlTextReaderPtr reader, struct global_data *gdata,
 	 * has a logic error (in this case, session ID doesn't match parent's).
 	 */
 	if (strcmp(expected_session, session_id) != 0) {
-		pr_info("File session id [%s] doesn't match parent's session id [%s]",
+		pr_val_info("File session id [%s] doesn't match parent's session id [%s]",
 		    expected_session, session_id);
 		error = EINVAL;
 		goto return_val;
@@ -443,7 +435,7 @@ parse_global_data(xmlTextReaderPtr reader, struct global_data *gdata,
 
 	/* ...and the serial must match to what's expected at the parent */
 	if (serial != expected_serial) {
-		pr_info("File serial '%lu' doesn't match expected serial '%lu'",
+		pr_val_info("File serial '%lu' doesn't match expected serial '%lu'",
 		    serial, expected_serial);
 		error = EINVAL;
 	}
@@ -514,7 +506,7 @@ parse_publish(xmlTextReaderPtr reader, bool parse_hash, bool hash_required,
 
 	/* Read the text */
 	if (xmlTextReaderRead(reader) != 1) {
-		error = pr_err("Couldn't read publish content of element '%s'",
+		error = pr_val_err("Couldn't read publish content of element '%s'",
 		    tmp->doc_data.uri);
 		goto release_tmp;
 	}
@@ -540,7 +532,7 @@ parse_publish(xmlTextReaderPtr reader, bool parse_hash, bool hash_required,
 		    tmp->doc_data.hash_len);
 		uri_refput(uri);
 		if (error != 0) {
-			pr_info("Hash of base64 decoded element from URI '%s' doesn't match <publish> element hash",
+			pr_val_info("Hash of base64 decoded element from URI '%s' doesn't match <publish> element hash",
 			    tmp->doc_data.uri);
 			error = EINVAL;
 			goto release_base64;
@@ -624,7 +616,7 @@ write_from_uri(char const *location, unsigned char *content, size_t content_len,
 	if (written != content_len) {
 		uri_refput(uri);
 		file_close(out);
-		return pr_err("Couldn't write bytes to file %s",
+		return pr_val_err("Couldn't write bytes to file %s",
 		    uri_get_local(uri));
 	}
 
@@ -796,11 +788,11 @@ order_notification_deltas(struct rdr_notification_ctx *ctx)
 			continue;
 
 		if (error == -EINVAL)
-			return pr_err("Serial '%lu' at delta elements isn't part of a contiguous list of serials.",
+			return pr_val_err("Serial '%lu' at delta elements isn't part of a contiguous list of serials.",
 			    (*ptr)->serial);
 
 		if (error == -EEXIST)
-			return pr_err("Duplicated serial '%lu' at delta elements.",
+			return pr_val_err("Duplicated serial '%lu' at delta elements.",
 			    (*ptr)->serial);
 
 		return error;
@@ -815,7 +807,7 @@ order_notification_deltas(struct rdr_notification_ctx *ctx)
 	 * If all expected elements are set, everything is ok.
 	 */
 	if (!deltas_head_values_set(ctx->notification->deltas_list))
-		return pr_err("Deltas listed don't have a contiguous sequence of serial numbers");
+		return pr_val_err("Deltas listed don't have a contiguous sequence of serial numbers");
 
 	return 0;
 }
@@ -845,7 +837,7 @@ xml_read_notification(xmlTextReaderPtr reader, void *arg)
 			/* Init context for deltas and snapshot */
 			rdr_notification_ctx_init(ctx);
 		} else {
-			return pr_err("Unexpected '%s' element", name);
+			return pr_val_err("Unexpected '%s' element", name);
 		}
 		break;
 	case XML_READER_TYPE_END_ELEMENT:
@@ -918,7 +910,7 @@ xml_read_snapshot(xmlTextReaderPtr reader, void *arg)
 			    ctx->parent->global_data.session_id,
 			    ctx->parent->global_data.serial);
 		else
-			return pr_err("Unexpected '%s' element", name);
+			return pr_val_err("Unexpected '%s' element", name);
 
 		if (error)
 			return error;
@@ -983,7 +975,7 @@ xml_read_delta(xmlTextReaderPtr reader, void *arg)
 			    ctx->parent->global_data.session_id,
 			    ctx->expected_serial);
 		else
-			return pr_err("Unexpected '%s' element", name);
+			return pr_val_err("Unexpected '%s' element", name);
 
 		if (error)
 			return error;
@@ -996,9 +988,9 @@ xml_read_delta(xmlTextReaderPtr reader, void *arg)
 }
 
 static int
-parse_delta(struct rpki_uri *uri, struct delta_head *parents_data, void *arg)
+parse_delta(struct rpki_uri *uri, struct delta_head *parents_data,
+    struct proc_upd_args *args)
 {
-	struct proc_upd_args *args = arg;
 	struct rdr_delta_ctx ctx;
 	struct delta *delta;
 	struct doc_data *expected_data;
@@ -1032,23 +1024,24 @@ pop_fnstack:
 static int
 process_delta(struct delta_head *delta_head, void *arg)
 {
+	struct proc_upd_args *args = arg;
 	struct rpki_uri *uri;
 	struct doc_data *head_data;
 	int error;
 
 	head_data = &delta_head->doc_data;
 
-	pr_debug("Processing delta '%s'.", delta_head->doc_data.uri);
+	pr_val_debug("Processing delta '%s'.", delta_head->doc_data.uri);
 	error = uri_create_https_str(&uri, head_data->uri,
 	    strlen(head_data->uri));
 	if (error)
 		return error;
 
-	error = download_file(uri, 0);
+	error = download_file(uri, 0, args->log_operation);
 	if (error)
 		goto release_uri;
 
-	error = parse_delta(uri, delta_head, arg);
+	error = parse_delta(uri, delta_head, args);
 
 	delete_from_uri(uri, NULL);
 	/* Error 0 its ok */
@@ -1066,7 +1059,7 @@ release_uri:
  * 'If-Modified-Since' header, return 0 and set @result to NULL.
  */
 int
-rrdp_parse_notification(struct rpki_uri *uri,
+rrdp_parse_notification(struct rpki_uri *uri, bool log_operation,
     struct update_notification **result)
 {
 	long last_update;
@@ -1075,16 +1068,20 @@ rrdp_parse_notification(struct rpki_uri *uri,
 	if (uri == NULL || uri_is_rsync(uri))
 		pr_crit("Wrong call, trying to parse a non HTTPS URI");
 
-	pr_debug("Processing notification '%s'.", uri_get_global(uri));
+	pr_val_debug("Processing notification '%s'.", uri_get_global(uri));
 	last_update = 0;
 	error = db_rrdp_uris_get_last_update(uri_get_global(uri), &last_update);
 	if (error && error != -ENOENT)
 		return error;
 
-	error = download_file(uri, last_update);
+	error = download_file(uri, last_update, log_operation);
 	if (error < 0)
 		return error;
 
+	/* Request error, stop processing to handle as such */
+	if (error == EREQFAILED)
+		return error;
+
 	/*
 	 * Mark as visited, if it doesn't exists yet, there's no problem since
 	 * this is probably the first time is visited (first run), so it will
@@ -1093,7 +1090,7 @@ rrdp_parse_notification(struct rpki_uri *uri,
 	vis_err = db_rrdp_uris_set_request_status(uri_get_global(uri),
 	    RRDP_URI_REQ_VISITED);
 	if (vis_err && vis_err != -ENOENT)
-		return pr_err("Couldn't mark '%s' as visited",
+		return pr_val_err("Couldn't mark '%s' as visited",
 		    uri_get_global(uri));
 
 	/* No updates yet */
@@ -1116,7 +1113,7 @@ rrdp_parse_notification(struct rpki_uri *uri,
 
 int
 rrdp_parse_snapshot(struct update_notification *parent,
-    struct visited_uris *visited_uris)
+    struct visited_uris *visited_uris, bool log_operation)
 {
 	struct proc_upd_args args;
 	struct rpki_uri *uri;
@@ -1125,13 +1122,13 @@ rrdp_parse_snapshot(struct update_notification *parent,
 	args.parent = parent;
 	args.visited_uris = visited_uris;
 
-	pr_debug("Processing snapshot '%s'.", parent->snapshot.uri);
+	pr_val_debug("Processing snapshot '%s'.", parent->snapshot.uri);
 	error = uri_create_https_str(&uri, parent->snapshot.uri,
 	    strlen(parent->snapshot.uri));
 	if (error)
 		return error;
 
-	error = download_file(uri, 0);
+	error = download_file(uri, 0, log_operation);
 	if (error)
 		goto release_uri;
 
@@ -1146,12 +1143,14 @@ release_uri:
 
 int
 rrdp_process_deltas(struct update_notification *parent,
-    unsigned long cur_serial, struct visited_uris *visited_uris)
+    unsigned long cur_serial, struct visited_uris *visited_uris,
+    bool log_operation)
 {
 	struct proc_upd_args args;
 
 	args.parent = parent;
 	args.visited_uris = visited_uris;
+	args.log_operation = log_operation;
 
 	return deltas_head_for_each(parent->deltas_list,
 	    parent->global_data.serial, cur_serial, process_delta, &args);
