@@ -11,7 +11,8 @@
 #include "common.h"
 #include "config.h"
 #include "log.h"
-#include "str.h"
+#include "reqs_errors.h"
+#include "str_token.h"
 #include "thread_var.h"
 
 struct uri {
@@ -191,14 +192,22 @@ duplicate_fds(int fds[2][2])
 }
 
 static void
-handle_child_thread(struct rpki_uri *uri, bool is_ta, int fds[2][2])
+release_args(char **args, unsigned int size)
 {
-	/* THIS FUNCTION MUST NEVER RETURN!!! */
+	unsigned int i;
 
+	/* args[0] wasn't allocated */
+	for (i = 1; i < size + 1; i++)
+		free(args[i]);
+	free(args);
+}
+
+static int
+prepare_rsync(struct rpki_uri *uri, bool is_ta, char ***args, size_t *args_len)
+{
 	struct string_array const *config_args;
 	char **copy_args;
 	unsigned int i;
-	int error;
 
 	config_args = config_get_rsync_args(is_ta);
 	/*
@@ -208,7 +217,7 @@ handle_child_thread(struct rpki_uri *uri, bool is_ta, int fds[2][2])
 	 */
 	copy_args = calloc(config_args->length + 2, sizeof(char *));
 	if (copy_args == NULL)
-		exit(pr_enomem());
+		return pr_enomem();
 
 	copy_args[0] = config_get_rsync_program();
 	copy_args[config_args->length + 1] = NULL;
@@ -216,24 +225,36 @@ handle_child_thread(struct rpki_uri *uri, bool is_ta, int fds[2][2])
 	memcpy(copy_args + 1, config_args->array,
 	    config_args->length * sizeof(char *));
 
-	for (i = 1; i < config_args->length + 1; i++) {
-		if (strcmp(copy_args[i], "$REMOTE") == 0)
-			copy_args[i] = strdup(uri_get_global(uri));
-		else if (strcmp(copy_args[i], "$LOCAL") == 0)
-			copy_args[i] = strdup(uri_get_local(uri));
-		if (copy_args[i] == NULL)
-			exit(pr_enomem());
+	for (i = 0; i < config_args->length; i++) {
+		if (strcmp(config_args->array[i], "$REMOTE") == 0)
+			copy_args[i + 1] = strdup(uri_get_global(uri));
+		else if (strcmp(config_args->array[i], "$LOCAL") == 0)
+			copy_args[i + 1] = strdup(uri_get_local(uri));
+		else
+			copy_args[i + 1] = strdup(config_args->array[i]);
+		if (copy_args[i + 1] == NULL) {
+			release_args(copy_args, i);
+			return pr_enomem();
+		}
 	}
 
-	pr_debug("Executing RSYNC:");
-	for (i = 0; i < config_args->length + 1; i++)
-		pr_debug("    %s", copy_args[i]);
+	*args = copy_args;
+	*args_len = config_args->length;
+	return 0;
+}
+
+static void
+handle_child_thread(char **args, int fds[2][2])
+{
+	/* THIS FUNCTION MUST NEVER RETURN!!! */
+	int error;
 
 	duplicate_fds(fds);
 
-	execvp(copy_args[0], copy_args);
+	execvp(args[0], args);
 	error = errno;
-	pr_err("Could not execute the rsync command: %s",
+	/* Log directly to stderr, redirected by the pipes */
+	fprintf(stderr, "Could not execute the rsync command: %s\n",
 	    strerror(error));
 
 	/* https://stackoverflow.com/a/14493459/1735458 */
@@ -244,14 +265,18 @@ static int
 create_pipes(int fds[2][2])
 {
 	if (pipe(fds[0]) == -1)
-		return -pr_errno(errno, "Piping rsync stderr");
-	if (pipe(fds[1]) == -1)
-		return -pr_errno(errno, "Piping rsync stdout");
+		return -pr_op_errno(errno, "Piping rsync stderr");
+	if (pipe(fds[1]) == -1) {
+		/* Close pipe previously created */
+		close(fds[0][0]);
+		close(fds[0][1]);
+		return -pr_op_errno(errno, "Piping rsync stdout");
+	}
 	return 0;
 }
 
 static void
-log_buffer(char const *buffer, ssize_t read, int type)
+log_buffer(char const *buffer, ssize_t read, int type, bool log_operation)
 {
 #define PRE_RSYNC "[RSYNC exec]: "
 	char *cpy, *cur, *tmp;
@@ -272,10 +297,13 @@ log_buffer(char const *buffer, ssize_t read, int type)
 			cur = tmp + 1;
 			continue;
 		}
-		if (type == 0)
-			pr_err(PRE_RSYNC "%s", cur);
-		else
-			pr_info(PRE_RSYNC "%s", cur);
+		if (type == 0) {
+			if (log_operation)
+				pr_op_err(PRE_RSYNC "%s", cur);
+			pr_val_err(PRE_RSYNC "%s", cur);
+		} else {
+			pr_val_info(PRE_RSYNC "%s", cur);
+		}
 		cur = tmp + 1;
 	}
 	free(cpy);
@@ -283,7 +311,7 @@ log_buffer(char const *buffer, ssize_t read, int type)
 }
 
 static int
-read_pipe(int fd_pipe[2][2], int type)
+read_pipe(int fd_pipe[2][2], int type, bool log_operation)
 {
 	char buffer[4096];
 	ssize_t count;
@@ -293,95 +321,141 @@ read_pipe(int fd_pipe[2][2], int type)
 		if (count == -1) {
 			if (errno == EINTR)
 				continue;
-			return -pr_errno(errno, "Reading rsync buffer");
+			close(fd_pipe[type][0]); /* Close read end */
+			return -pr_val_errno(errno, "Reading rsync buffer");
 		}
 		if (count == 0)
 			break;
 
-		log_buffer(buffer, count, type);
+		log_buffer(buffer, count, type, log_operation);
 	}
-	close(fd_pipe[type][0]);
+	close(fd_pipe[type][0]); /* Close read end */
 	return 0;
 }
 
+/*
+ * Read the piped output from the child, assures that all pipes are closed on
+ * success and on error.
+ */
 static int
-read_pipes(int fds[2][2])
+read_pipes(int fds[2][2], bool log_operation)
 {
 	int error;
 
-	/* Won't be needed */
+	/* Won't be needed (sterr/stdout write ends) */
 	close(fds[0][1]);
 	close(fds[1][1]);
 
 	/* stderr pipe */
-	error = read_pipe(fds, 0);
-	if (error)
+	error = read_pipe(fds, 0, log_operation);
+	if (error) {
+		/* Close the other pipe pending to read */
+		close(fds[1][0]);
 		return error;
+	}
 
-	/* stdout pipe */
-	return read_pipe(fds, 1);
+	/* stdout pipe, always logs to info */
+	return read_pipe(fds, 1, true);
 }
 
 /*
  * Downloads the @uri->global file into the @uri->local path.
  */
 static int
-do_rsync(struct rpki_uri *uri, bool is_ta)
+do_rsync(struct rpki_uri *uri, bool is_ta, bool log_operation)
 {
 	/* Descriptors to pipe stderr (first element) and stdout (second) */
+	char **args;
+	size_t args_len;
 	int fork_fds[2][2];
 	pid_t child_pid;
 	unsigned int retries;
+	unsigned int i;
 	int child_status;
 	int error;
 
+	/* Prepare everything for the child exec */
+	args = NULL;
+	args_len = 0;
+	error = prepare_rsync(uri, is_ta, &args, &args_len);
+	if (error)
+		return error;
+
+	pr_val_debug("Executing RSYNC:");
+	for (i = 0; i < args_len + 1; i++)
+		pr_val_debug("    %s", args[i]);
+
 	retries = 0;
 	do {
 		child_status = 0;
 		error = create_dir_recursive(uri_get_local(uri));
 		if (error)
-			return error;
+			goto release_args;
 
 		error = create_pipes(fork_fds);
 		if (error)
-			return error;
+			goto release_args;
+
+		/* Flush output (avoid locks between father and child) */
+		log_flush();
 
 		/* We need to fork because execvp() magics the thread away. */
 		child_pid = fork();
 		if (child_pid == 0) {
-			/* This code is run by the child. */
-			handle_child_thread(uri, is_ta, fork_fds);
+			/*
+			 * This code is run by the child, and should try to
+			 * call execvp() as soon as possible.
+			 *
+			 * Refer to
+			 * https://pubs.opengroup.org/onlinepubs/9699919799/functions/fork.html
+			 * "{..} to avoid errors, the child process may only
+			 * execute async-signal-safe operations until such time
+			 * as one of the exec functions is called."
+			 */
+			handle_child_thread(args, fork_fds);
+		}
+		if (child_pid < 0) {
+			pr_op_errno(errno, "Couldn't fork to execute rsync");
+			error = errno;
+			/* Close all ends from the created pipes */
+			close(fork_fds[0][0]);
+			close(fork_fds[1][0]);
+			close(fork_fds[0][1]);
+			close(fork_fds[1][1]);
+			goto release_args;
 		}
 
 		/* This code is run by us. */
-		error = read_pipes(fork_fds);
+		error = read_pipes(fork_fds, log_operation);
 		if (error)
-			return error;
+			kill(child_pid, SIGCHLD); /* Stop the child */
 
 		error = waitpid(child_pid, &child_status, 0);
 		do {
 			if (error == -1) {
 				error = errno;
-				pr_err("The rsync sub-process returned error %d (%s)",
+				pr_op_err("The rsync sub-process returned error %d (%s)",
 				    error, strerror(error));
 				if (child_status > 0)
 					break;
-				return error;
+				goto release_args;
 			}
 		} while (0);
 
 		if (WIFEXITED(child_status)) {
 			/* Happy path (but also sad path sometimes). */
 			error = WEXITSTATUS(child_status);
-			pr_debug("Child terminated with error code %d.", error);
+			pr_val_debug("Child terminated with error code %d.", error);
 			if (!error)
-				return 0;
+				goto release_args;
+
 			if (retries == config_get_rsync_retry_count()) {
-				pr_info("Max RSYNC retries (%u) reached on '%s', won't retry again.",
+				pr_val_warn("Max RSYNC retries (%u) reached on '%s', won't retry again.",
 				    retries, uri_get_global(uri));
-				return error;
+				error = EREQFAILED;
+				goto release_args;
 			}
-			pr_info("Retrying RSYNC '%s' in %u seconds, %u attempts remaining.",
+			pr_val_warn("Retrying RSYNC '%s' in %u seconds, %u attempts remaining.",
 			    uri_get_global(uri),
 			    config_get_rsync_retry_interval(),
 			    config_get_rsync_retry_count() - retries);
@@ -392,27 +466,64 @@ do_rsync(struct rpki_uri *uri, bool is_ta)
 		break;
 	} while (true);
 
+	release_args(args, args_len);
+
 	if (WIFSIGNALED(child_status)) {
 		switch (WTERMSIG(child_status)) {
 		case SIGINT:
-			pr_err("RSYNC was user-interrupted. Guess I'll interrupt myself too.");
+			pr_op_err("RSYNC was user-interrupted. Guess I'll interrupt myself too.");
 			break;
 		case SIGQUIT:
-			pr_err("RSYNC received a quit signal. Guess I'll quit as well.");
+			pr_op_err("RSYNC received a quit signal. Guess I'll quit as well.");
 			break;
 		case SIGKILL:
-			pr_err("Killed.");
+			pr_op_err("Killed.");
 			break;
 		default:
-			pr_err("The RSYNC was terminated by a signal [%d] I don't have a handler for. Dunno; guess I'll just die.",
+			pr_op_err("The RSYNC was terminated by a signal [%d] I don't have a handler for. Dunno; guess I'll just die.",
 			    WTERMSIG(child_status));
 			break;
 		}
 		return -EINTR; /* Meh? */
 	}
 
-	pr_err("The RSYNC command died in a way I don't have a handler for. Dunno; guess I'll die as well.");
+	pr_op_err("The RSYNC command died in a way I don't have a handler for. Dunno; guess I'll die as well.");
 	return -EINVAL;
+release_args:
+	/* The happy path also falls here */
+	release_args(args, args_len);
+	return error;
+}
+
+/*
+ * Returned values if the ancestor URI of @error_uri:
+ * 0 - didn't had a previous request error
+ * EEXIST - had a previous request error
+ * > 0 - nothing, just something bad happened
+ */
+static int
+ancestor_error(char const *error_uri, void *arg)
+{
+	struct rpki_uri *search = arg;
+	struct rpki_uri *req_err_uri;
+	int error;
+
+	req_err_uri = NULL;
+	error = uri_create_rsync_str(&req_err_uri, error_uri,
+	    strlen(error_uri));
+	switch(error) {
+	case 0:
+		break;
+	case ENOTRSYNC:
+		return 0;
+	default:
+		return ENSURE_NEGATIVE(error);
+	}
+
+	error = is_descendant(req_err_uri, search) ? EEXIST : 0;
+
+	free(req_err_uri);
+	return error;
 }
 
 /**
@@ -434,6 +545,7 @@ download_files(struct rpki_uri *requested_uri, bool is_ta, bool force)
 	struct validation *state;
 	struct uri_list *visited_uris;
 	struct rpki_uri *rsync_uri;
+	bool to_op_log;
 	int error;
 
 	if (!config_get_rsync_enabled())
@@ -446,26 +558,51 @@ download_files(struct rpki_uri *requested_uri, bool is_ta, bool force)
 	visited_uris = validation_rsync_visited_uris(state);
 
 	if (!force && is_already_downloaded(requested_uri, visited_uris)) {
-		pr_debug("No need to redownload '%s'.",
-		    uri_get_printable(requested_uri));
+		pr_val_debug("No need to redownload '%s'.",
+		    uri_val_get_printable(requested_uri));
 		return 0;
 	}
 
 	if (!force)
 		error = get_rsync_uri(requested_uri, is_ta, &rsync_uri);
-	else
+	else {
+		/* Validate if the ancestor URI error'd */
+		error = reqs_errors_foreach(ancestor_error, requested_uri);
+		if (error < 0)
+			return error;
+		/* Return the requests error'd code */
+		if (error == EEXIST)
+			return EREQFAILED;
 		error = handle_strict_strategy(requested_uri, &rsync_uri);
+	}
 
 	if (error)
 		return error;
 
-	pr_debug("Going to RSYNC '%s'.", uri_get_printable(rsync_uri));
+	pr_val_debug("Going to RSYNC '%s'.", uri_val_get_printable(rsync_uri));
 
-	/* Don't store when "force" and if its already downloaded */
-	error = do_rsync(rsync_uri, is_ta);
-	if (!error &&
-	    !(force && is_already_downloaded(rsync_uri, visited_uris)))
+	to_op_log = reqs_errors_log_uri(uri_get_global(requested_uri));
+	error = do_rsync(rsync_uri, is_ta, to_op_log);
+	switch(error) {
+	case 0:
+		/* Don't store when "force" and if its already downloaded */
+		if (!(force && is_already_downloaded(rsync_uri, visited_uris)))
+			error = mark_as_downloaded(rsync_uri, visited_uris);
+		reqs_errors_rem_uri(uri_get_global(requested_uri));
+		break;
+	case EREQFAILED:
+		/* All attempts failed, avoid future requests */
+		error = reqs_errors_add_uri(uri_get_global(requested_uri));
+		if (error)
+			break;
 		error = mark_as_downloaded(rsync_uri, visited_uris);
+		/* Everything went ok? Return the original error */
+		if (!error)
+			error = EREQFAILED;
+		break;
+	default:
+		break;
+	}
 
 	uri_refput(rsync_uri);
 	return error;
