@@ -60,7 +60,9 @@ struct state {
 	serial_t next_serial;
 	uint16_t v0_session_id;
 	uint16_t v1_session_id;
-} state;
+};
+
+static struct state state;
 
 /** Read/write lock, which protects @state and its inhabitants. */
 static pthread_rwlock_t state_lock;
@@ -77,6 +79,7 @@ deltagroup_cleanup(struct delta_group *group)
 int
 vrps_init(void)
 {
+	time_t now;
 	int error;
 
 	state.base = NULL;
@@ -91,7 +94,12 @@ vrps_init(void)
 	state.next_serial = START_SERIAL;
 
 	/* Get the bits that'll fit in session_id */
-	state.v0_session_id = time(NULL) & 0xFFFF;
+	now = 0;
+	error = get_current_time(&now);
+	if (error)
+		goto release_deltas;
+	state.v0_session_id = now & 0xFFFF;
+
 	/* Minus 1 to prevent same ID */
 	state.v1_session_id = (state.v0_session_id != 0)
 	    ? (state.v0_session_id - 1)
@@ -101,13 +109,13 @@ vrps_init(void)
 
 	error = pthread_rwlock_init(&state_lock, NULL);
 	if (error) {
-		error = pr_errno(error, "state pthread_rwlock_init() errored");
+		error = pr_op_errno(error, "state pthread_rwlock_init() errored");
 		goto release_deltas;
 	}
 
 	error = pthread_rwlock_init(&table_lock, NULL);
 	if (error) {
-		error = pr_errno(error, "table pthread_rwlock_init() errored");
+		error = pr_op_errno(error, "table pthread_rwlock_init() errored");
 		goto release_state_lock;
 	}
 
@@ -182,7 +190,6 @@ __perform_standalone_validation(struct db_table **result)
 
 	error = perform_standalone_validation(db);
 	if (error) {
-		terminate_standalone_validation();
 		db_table_destroy(db);
 		return error;
 	}
@@ -289,13 +296,11 @@ __vrps_update(bool *changed)
 	if (error)
 		return error;
 
-	rwlock_write_lock(&state_lock);
-
 	error = slurm_apply(&new_base, &state.slurm);
-	if (error) {
-		rwlock_unlock(&state_lock);
+	if (error)
 		goto revert_base;
-	}
+
+	rwlock_write_lock(&state_lock);
 
 	if (state.base != NULL) {
 		error = compute_deltas(state.base, new_base, &deltas);
@@ -382,15 +387,15 @@ vrps_update(bool *changed)
 	 * This wrapper is mainly for log informational data, so if there's no
 	 * need don't do unnecessary calls
 	 */
-	if (!log_info_enabled())
+	if (!log_op_info_enabled())
 		return __vrps_update(changed);
 
-	pr_info("Starting validation.");
+	pr_op_info("Starting validation.");
 	serial = START_SERIAL;
 	if (config_get_mode() == SERVER) {
 		error = get_last_serial_number(&serial);
 		if (!error)
-			pr_info("- Current serial number is %u.", serial);
+			pr_op_info("- Current serial number is %u.", serial);
 	}
 
 	time(&start);
@@ -398,29 +403,29 @@ vrps_update(bool *changed)
 	time(&finish);
 	exec_time = finish - start;
 
-	pr_info("Validation finished:");
+	pr_op_info("Validation finished:");
 	rwlock_read_lock(&state_lock);
 	do {
 		if (state.base == NULL) {
 			rwlock_unlock(&state_lock);
-			pr_info("- Valid Prefixes: 0");
-			pr_info("- Valid Router Keys: 0");
+			pr_op_info("- Valid Prefixes: 0");
+			pr_op_info("- Valid Router Keys: 0");
 			if (config_get_mode() == SERVER)
-				pr_info("- No serial number.");
+				pr_op_info("- No serial number.");
 			break;
 		}
 
-		pr_info("- Valid Prefixes: %u", db_table_roa_count(state.base));
-		pr_info("- Valid Router Keys: %u",
+		pr_op_info("- Valid Prefixes: %u", db_table_roa_count(state.base));
+		pr_op_info("- Valid Router Keys: %u",
 		    db_table_router_key_count(state.base));
 		if (config_get_mode() == SERVER) {
-			pr_info("- %s serial number is %u.",
+			pr_op_info("- %s serial number is %u.",
 			    serial == state.next_serial - 1 ? "Current" : "New",
 			    state.next_serial - 1);
 		}
 		rwlock_unlock(&state_lock);
 	} while(0);
-	pr_info("- Real execution time: %ld secs.", exec_time);
+	pr_op_info("- Real execution time: %ld secs.", exec_time);
 
 	return error;
 }
