@@ -16,28 +16,28 @@ pdutype2str(enum pdu_type type)
 {
 	switch (type) {
 	case PDU_TYPE_SERIAL_NOTIFY:
-		return "Serial Notify";
+		return "Serial Notify PDU";
 	case PDU_TYPE_SERIAL_QUERY:
-		return "Serial Query";
+		return "Serial Query PDU";
 	case PDU_TYPE_RESET_QUERY:
-		return "Reset Query";
+		return "Reset Query PDU";
 	case PDU_TYPE_CACHE_RESPONSE:
-		return "Cache Response";
+		return "Cache Response PDU";
 	case PDU_TYPE_IPV4_PREFIX:
-		return "IPv4 Prefix";
+		return "IPv4 Prefix PDU";
 	case PDU_TYPE_IPV6_PREFIX:
-		return "IPv6 Prefix";
+		return "IPv6 Prefix PDU";
 	case PDU_TYPE_END_OF_DATA:
-		return "End of Data";
+		return "End of Data PDU";
 	case PDU_TYPE_CACHE_RESET:
-		return "Cache Reset";
+		return "Cache Reset PDU";
 	case PDU_TYPE_ROUTER_KEY:
-		return "Router Key";
+		return "Router Key PDU";
 	case PDU_TYPE_ERROR_REPORT:
-		return "Error Report";
+		return "Error Report PDU";
 	}
 
-	return "(unknown)";
+	return "unknown PDU";
 }
 
 static int
@@ -126,9 +126,9 @@ pdu_load(int fd, struct sockaddr_storage *client_addr,
 		return error;
 
 
-	if (log_debug_enabled()) {
+	if (log_op_debug_enabled()) {
 		char buffer[INET6_ADDRSTRLEN];
-		pr_debug("Received a %s PDU from %s.",
+		pr_op_debug("Received a %s from %s.",
 		    pdutype2str(header.pdu_type),
 		    sockaddr2str(client_addr, buffer));
 	}
@@ -355,7 +355,7 @@ DEFINE_METADATA(cache_reset, free);
 DEFINE_METADATA(router_key, free);
 DEFINE_METADATA(error_report, error_report_destroy);
 
-struct pdu_metadata const *const pdu_metadatas[] = {
+static struct pdu_metadata const *const pdu_metadatas[] = {
 	/* 0 */  &serial_notify_meta,
 	/* 1 */  &serial_query_meta,
 	/* 2 */  &reset_query_meta,
