@@ -1,14 +1,17 @@
 #include "rtr.h"
 
 #include <errno.h>
+#include <fcntl.h>
 #include <netdb.h>
 #include <pthread.h>
+#include <signal.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <arpa/inet.h>
+#include <sys/queue.h>
 
 #include "config.h"
 #include "clients.h"
@@ -18,17 +21,27 @@
 #include "rtr/pdu.h"
 #include "rtr/db/vrps.h"
 
+/* Parameters for each thread that handles client connections */
 struct thread_param {
 	int fd;
 	pthread_t tid;
 	struct sockaddr_storage addr;
 };
 
+/* Parameters for each thread that binds to a server address/socket */
+struct fd_node {
+	int id;
+	pthread_t tid;
+	SLIST_ENTRY(fd_node) next;
+};
+
+/* List of server sockets */
+SLIST_HEAD(server_fds, fd_node);
+
 static int
-init_addrinfo(struct addrinfo **result)
+init_addrinfo(char const *hostname, char const *service,
+    struct addrinfo **result)
 {
-	char const *hostname;
-	char const *service;
 	char *tmp;
 	struct addrinfo hints;
 	unsigned long parsed, port;
@@ -39,15 +52,12 @@ init_addrinfo(struct addrinfo **result)
 	/* hints.ai_socktype = SOCK_DGRAM; */
 	hints.ai_flags |= AI_PASSIVE;
 
-	hostname = config_get_server_address();
-	service = config_get_server_port();
-
 	if (hostname != NULL)
 		hints.ai_flags |= AI_CANONNAME;
 
 	error = getaddrinfo(hostname, service, &hints, result);
 	if (error) {
-		pr_err("Could not infer a bindable address out of address '%s' and port '%s': %s",
+		pr_op_err("Could not infer a bindable address out of address '%s' and port '%s': %s",
 		    (hostname != NULL) ? hostname : "any", service,
 		    gai_strerror(error));
 		return error;
@@ -70,7 +80,7 @@ init_addrinfo(struct addrinfo **result)
 	port = (unsigned char)((*result)->ai_addr->sa_data[0]) << 8;
 	port += (unsigned char)((*result)->ai_addr->sa_data[1]);
 	if (parsed != port)
-		return pr_err("Service port %s is out of range (max value is %d)",
+		return pr_op_err("Service port %s is out of range (max value is %d)",
 		    service, USHRT_MAX);
 
 	return 0;
@@ -81,11 +91,12 @@ init_addrinfo(struct addrinfo **result)
  * from the clients.
  */
 static int
-create_server_socket(int *result)
+create_server_socket(char const *hostname, char const *service, int *result)
 {
 	struct addrinfo *addrs;
 	struct addrinfo *addr;
 	unsigned long port;
+	int flags;
 	int reuse;
 	int fd; /* "file descriptor" */
 	int error;
@@ -93,38 +104,58 @@ create_server_socket(int *result)
 	*result = 0; /* Shuts up gcc */
 	reuse = 1;
 
-	error = init_addrinfo(&addrs);
+	error = init_addrinfo(hostname, service, &addrs);
 	if (error)
 		return error;
 
-	for (addr = addrs; addr != NULL; addr = addr->ai_next) {
-		pr_info(
+	if (addrs != NULL)
+		pr_op_info(
 		    "Attempting to bind socket to address '%s', port '%s'.",
-		    (addr->ai_canonname != NULL) ? addr->ai_canonname : "any",
-		    config_get_server_port());
+		    (addrs->ai_canonname != NULL) ? addrs->ai_canonname : "any",
+		    service);
 
+	for (addr = addrs; addr != NULL; addr = addr->ai_next) {
 		fd = socket(addr->ai_family, SOCK_STREAM, 0);
 		if (fd < 0) {
-			pr_errno(errno, "socket() failed");
+			pr_op_errno(errno, "socket() failed");
+			continue;
+		}
+
+		flags = fcntl(fd, F_GETFL);
+		if (flags == -1) {
+			pr_op_errno(errno, "fcntl() to get flags failed");
+			close(fd);
+			continue;
+		}
+
+		/* Non-block to allow listening on all server sockets */
+		flags |= O_NONBLOCK;
+
+		if (fcntl(fd, F_SETFL, flags) == -1) {
+			pr_op_errno(errno, "fcntl() to set flags failed");
+			close(fd);
 			continue;
 		}
 
 		/* enable SO_REUSEADDR */
 		if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &reuse,
 		    sizeof(int)) < 0) {
-			pr_errno(errno, "setsockopt(SO_REUSEADDR) failed");
+			pr_op_errno(errno, "setsockopt(SO_REUSEADDR) failed");
+			close(fd);
 			continue;
 		}
 
 		/* enable SO_REUSEPORT */
 		if (setsockopt(fd, SOL_SOCKET, SO_REUSEPORT, &reuse,
 		    sizeof(int)) < 0) {
-			pr_errno(errno, "setsockopt(SO_REUSEPORT) failed");
+			pr_op_errno(errno, "setsockopt(SO_REUSEPORT) failed");
+			close(fd);
 			continue;
 		}
 
 		if (bind(fd, addr->ai_addr, addr->ai_addrlen) < 0) {
-			pr_errno(errno, "bind() failed");
+			pr_op_errno(errno, "bind() failed");
+			close(fd);
 			continue;
 		}
 
@@ -132,12 +163,12 @@ create_server_socket(int *result)
 		if (error) {
 			close(fd);
 			freeaddrinfo(addrs);
-			return pr_errno(errno, "getsockname() failed");
+			return pr_op_errno(errno, "getsockname() failed");
 		}
 
 		port = (unsigned char)(addr->ai_addr->sa_data[0]) << 8;
 		port += (unsigned char)(addr->ai_addr->sa_data[1]);
-		pr_info("Success; bound to address '%s', port '%ld'.",
+		pr_op_info("Success; bound to address '%s', port '%ld'.",
 		    (addr->ai_canonname != NULL) ? addr->ai_canonname : "any",
 		    port);
 		freeaddrinfo(addrs);
@@ -146,7 +177,143 @@ create_server_socket(int *result)
 	}
 
 	freeaddrinfo(addrs);
-	return pr_err("None of the addrinfo candidates could be bound.");
+	return pr_op_err("None of the addrinfo candidates could be bound.");
+}
+
+static int
+fd_node_create(struct fd_node **result)
+{
+	struct fd_node *node;
+
+	node = malloc(sizeof(struct fd_node));
+	if (node == NULL)
+		return pr_enomem();
+
+	node->id = -1;
+	node->tid = -1;
+
+	*result = node;
+	return 0;
+}
+
+static int
+server_fd_add(struct server_fds *fds, char const *address, char const *service)
+{
+	struct fd_node *node;
+	int error;
+
+	node = NULL;
+	error = fd_node_create(&node);
+	if (error)
+		return error;
+
+	error = create_server_socket(address, service, &node->id);
+	if (error) {
+		free(node);
+		return error;
+	}
+
+	SLIST_INSERT_HEAD(fds, node, next);
+	pr_op_debug("Created server socket with FD %d.", node->id);
+	return 0;
+}
+
+static void
+server_fd_cleanup(struct server_fds *fds)
+{
+	struct fd_node *fd;
+
+	while (!SLIST_EMPTY(fds)) {
+		fd = fds->slh_first;
+		SLIST_REMOVE_HEAD(fds, next);
+		close(fd->id);
+		free(fd);
+	}
+}
+
+static int
+parse_address(char const *full_address, char const *default_service,
+    char **address, char **service)
+{
+	char *ptr;
+	char *tmp_addr;
+	char *tmp_serv;
+	size_t tmp_addr_len;
+
+	ptr = strrchr(full_address, '#');
+	if (ptr == NULL) {
+		tmp_addr = strdup(full_address);
+		if (tmp_addr == NULL)
+			return pr_enomem();
+
+		tmp_serv = strdup(default_service);
+		if (tmp_serv == NULL) {
+			free(tmp_addr);
+			return pr_enomem();
+		}
+		*address = tmp_addr;
+		*service = tmp_serv;
+		return 0;
+	}
+
+	if (*(ptr + 1) == '\0')
+		return pr_op_err("Invalid server address '%s', can't end with '#'",
+		    full_address);
+
+	tmp_addr_len = strlen(full_address) - strlen(ptr);
+	tmp_addr = malloc(tmp_addr_len + 1);
+	if (tmp_addr == NULL)
+		return pr_enomem();
+
+	memcpy(tmp_addr, full_address, tmp_addr_len);
+	tmp_addr[tmp_addr_len] = '\0';
+
+	tmp_serv = strdup(ptr + 1);
+	if (tmp_serv == NULL) {
+		free(tmp_addr);
+		return pr_enomem();
+	}
+
+	*address = tmp_addr;
+	*service = tmp_serv;
+	return 0;
+}
+
+static int
+create_server_sockets(struct server_fds *fds)
+{
+	struct string_array const *addresses;
+	char const *default_service;
+	char *address;
+	char *service;
+	unsigned int i;
+	int error;
+
+	default_service = config_get_server_port();
+	addresses = config_get_server_address();
+	if (addresses->length == 0)
+		return server_fd_add(fds, NULL, default_service);
+
+	for (i = 0; i < addresses->length; i++) {
+		address = NULL;
+		service = NULL;
+		error = parse_address(addresses->array[i], default_service,
+		    &address, &service);
+		if (error)
+			goto cleanup_fds;
+
+		error = server_fd_add(fds, address, service);
+		/* Always release them */
+		free(address);
+		free(service);
+		if (error)
+			goto cleanup_fds;
+	}
+
+	return 0;
+cleanup_fds:
+	server_fd_cleanup(fds);
+	return error;
 }
 
 enum verdict {
@@ -196,12 +363,12 @@ handle_accept_result(int client_fd, int err)
 		goto retry;
 #pragma GCC diagnostic pop
 
-	pr_info("Client connection attempt not accepted: %s. Quitting...",
+	pr_op_info("Client connection attempt not accepted: %s. Quitting...",
 	    strerror(err));
 	return VERDICT_EXIT;
 
 retry:
-	pr_info("Client connection attempt not accepted: %s. Retrying...",
+	pr_op_info("Client connection attempt not accepted: %s. Retrying...",
 	    strerror(err));
 	return VERDICT_RETRY;
 }
@@ -224,7 +391,7 @@ print_close_failure(int error, int fd)
 	    ? sockaddr2str(&sockaddr, buffer)
 	    : "(unknown)";
 
-	pr_errno(error, "close() failed on socket of client %s", addr_str);
+	pr_op_errno(error, "close() failed on socket of client %s", addr_str);
 }
 
 static void
@@ -238,7 +405,7 @@ static void
 print_client_addr(struct sockaddr_storage *addr, char const *action, int fd)
 {
 	char buffer[INET6_ADDRSTRLEN];
-	pr_info("Client %s [ID %d]: %s", action, fd,
+	pr_op_info("Client %s [ID %d]: %s", action, fd,
 	    sockaddr2str(addr, buffer));
 }
 
@@ -284,65 +451,114 @@ client_thread_cb(void *arg)
 	return NULL;
 }
 
+static void
+init_fdset(struct server_fds *fds, fd_set *fdset)
+{
+	struct fd_node *node;
+
+	FD_ZERO (fdset);
+	SLIST_FOREACH(node, fds, next)
+		FD_SET (node->id, fdset);
+
+}
+
 /*
  * Waits for client connections and spawns threads to handle them.
  */
 static int
-handle_client_connections(int server_fd)
+handle_client_connections(struct server_fds *fds)
 {
+	struct fd_node *head_node;
+	struct sigaction ign;
 	struct sockaddr_storage client_addr;
 	struct thread_param *param;
+	struct timeval select_time;
 	socklen_t sizeof_client_addr;
+	fd_set readfds;
+	int last_server_fd;
 	int client_fd;
+	int fd;
 	int error;
 
-	listen(server_fd, config_get_server_queue());
+	/* Ignore SIGPIPES, they're handled apart */
+	ign.sa_handler = SIG_IGN;
+	ign.sa_flags = 0;
+	sigemptyset(&ign.sa_mask);
+	sigaction(SIGPIPE, &ign, NULL);
+
+	head_node = SLIST_FIRST(fds);
+	last_server_fd = head_node->id;
+
+	SLIST_FOREACH(head_node, fds, next) {
+		error = listen(head_node->id, config_get_server_queue());
+		if (error)
+			return pr_op_errno(errno,
+			    "Couldn't listen on server socket.");
+	}
 
 	sizeof_client_addr = sizeof(client_addr);
 
-	pr_debug("Waiting for client connections...");
+	pr_op_debug("Waiting for client connections at server...");
 	do {
-		client_fd = accept(server_fd, (struct sockaddr *) &client_addr,
-		    &sizeof_client_addr);
-		switch (handle_accept_result(client_fd, errno)) {
-		case VERDICT_SUCCESS:
-			break;
-		case VERDICT_RETRY:
-			continue;
-		case VERDICT_EXIT:
-			return -EINVAL;
-		}
+		/* Look for connections every .2 seconds*/
+		select_time.tv_sec = 0;
+		select_time.tv_usec = 200000;
 
-		print_client_addr(&client_addr, "accepted", client_fd);
+		init_fdset(fds, &readfds);
 
-		/*
-		 * Note: My gut says that errors from now on (even the unknown
-		 * ones) should be treated as temporary; maybe the next
-		 * accept() will work.
-		 * So don't interrupt the thread when this happens.
-		 */
-
-		param = malloc(sizeof(struct thread_param));
-		if (param == NULL) {
-			/* No error response PDU on memory allocation. */
-			pr_err("Couldn't create thread parameters struct");
-			close(client_fd);
+		if (select(last_server_fd + 1, &readfds, NULL, NULL,
+		    &select_time) == -1) {
+			pr_op_errno(errno, "Monitoring server sockets");
 			continue;
 		}
-		param->fd = client_fd;
-		param->addr = client_addr;
 
-		error = pthread_create(&param->tid, NULL,
-		    client_thread_cb, param);
-		if (error && error != EAGAIN)
-			/* Error with min RTR version */
-			err_pdu_send_internal_error(client_fd, RTR_V0);
-		if (error) {
-			pr_errno(error, "Could not spawn the client's thread");
-			close(client_fd);
-			free(param);
+		for (fd = 0; fd < (last_server_fd + 1); fd++) {
+			if (!FD_ISSET (fd, &readfds))
+				continue;
+
+			/* Accept the connection */
+			client_fd = accept(fd, (struct sockaddr *) &client_addr,
+			    &sizeof_client_addr);
+			switch (handle_accept_result(client_fd, errno)) {
+			case VERDICT_SUCCESS:
+				break;
+			case VERDICT_RETRY:
+				continue;
+			case VERDICT_EXIT:
+				return -EINVAL;
+			}
+
+			print_client_addr(&client_addr, "accepted", client_fd);
+
+			/*
+			 * Note: My gut says that errors from now on (even the
+			 * unknown ones) should be treated as temporary; maybe
+			 * the next accept() will work.
+			 * So don't interrupt the thread when this happens.
+			 */
+
+			param = malloc(sizeof(struct thread_param));
+			if (param == NULL) {
+				/* No error PDU on memory allocation. */
+				pr_enomem();
+				close(client_fd);
+				continue;
+			}
+			param->fd = client_fd;
+			param->addr = client_addr;
+
+			error = pthread_create(&param->tid, NULL,
+			    client_thread_cb, param);
+			if (error && error != EAGAIN)
+				/* Error with min RTR version */
+				err_pdu_send_internal_error(client_fd, RTR_V0);
+			if (error) {
+				pr_op_errno(error,
+				    "Could not spawn the client's thread");
+				close(client_fd);
+				free(param);
+			}
 		}
-
 	} while (true);
 
 	return 0; /* Unreachable. */
@@ -386,7 +602,7 @@ int
 rtr_listen(void)
 {
 	bool changed;
-	int server_fd; /* "file descriptor" */
+	struct server_fds fds; /* "file descriptors" */
 	int error;
 
 	error = clients_db_init();
@@ -396,25 +612,26 @@ rtr_listen(void)
 	if (config_get_mode() == STANDALONE) {
 		error = vrps_update(&changed);
 		if (error)
-			pr_err("Error %d while trying to update the ROA database.",
+			pr_op_err("Error %d while trying to update the ROA database.",
 			    error);
 		goto revert_clients_db; /* Error 0 it's ok */
 	}
 
-	error = create_server_socket(&server_fd);
+	SLIST_INIT(&fds);
+	error = create_server_sockets(&fds);
 	if (error)
 		goto revert_clients_db;
 
 	error = updates_daemon_start();
 	if (error)
-		goto revert_server_socket;
+		goto revert_server_sockets;
 
-	error = handle_client_connections(server_fd);
+	error = handle_client_connections(&fds);
 
 	end_clients();
 	updates_daemon_destroy();
-revert_server_socket:
-	close(server_fd);
+revert_server_sockets:
+	server_fd_cleanup(&fds);
 revert_clients_db:
 	clients_db_destroy(join_thread, NULL);
 	return error;
