@@ -34,13 +34,9 @@
 
 #define CHECK_REQUIRED(element, name)					\
 	if (element == NULL)						\
-		return pr_err("SLURM member '%s' is required", name);
+		return pr_op_err("SLURM member '%s' is required", name);
 
-/* Context value, local to avoid forwarding the parameter */
-struct db_slurm *db;
-unsigned int cur_ctx;
-
-static int handle_json(json_t *, struct db_slurm *, unsigned int *);
+static int handle_json(json_t *, struct db_slurm *);
 
 /*
  * Try to parse the SLURM file(s)
@@ -51,7 +47,6 @@ slurm_parse(char const *location, void *arg)
 	struct slurm_parser_params *params;
 	json_t *json_root;
 	json_error_t json_error;
-	unsigned int ctx;
 	int error;
 
 	params = arg;
@@ -60,16 +55,14 @@ slurm_parse(char const *location, void *arg)
 	    &json_error);
 	if (json_root == NULL)
 		/* File wasn't read or has a content error */
-		return pr_err("SLURM JSON error on line %d, column %d: %s",
+		return pr_op_err("SLURM JSON error on line %d, column %d: %s",
 		    json_error.line, json_error.column, json_error.text);
 
-	ctx = params->cur_ctx;
-	error = handle_json(json_root, params->db_slurm, &ctx);
+	error = handle_json(json_root, params->db_slurm);
 	json_decref(json_root);
 	if (error)
 		return error; /* File exists, but has a syntax error */
 
-	params->cur_ctx = ctx;
 	return 0;
 }
 
@@ -83,7 +76,7 @@ set_asn(json_t *object, bool is_assertion, uint32_t *result, uint8_t *flag,
 	error = json_get_int(object, ASN, &int_tmp);
 	if (error == -ENOENT) {
 		if (is_assertion)
-			return pr_err("ASN is required");
+			return pr_op_err("ASN is required");
 		else
 			return 0; /* Optional for filters */
 	} else if (error)
@@ -91,7 +84,7 @@ set_asn(json_t *object, bool is_assertion, uint32_t *result, uint8_t *flag,
 
 	/* An underflow or overflow will be considered here */
 	if (int_tmp < 0 || UINT32_MAX < int_tmp)
-		return pr_err("ASN (%lld) is out of range [0 - %u].", int_tmp,
+		return pr_op_err("ASN (%lld) is out of range [0 - %u].", int_tmp,
 		    UINT32_MAX);
 	*flag = *flag | SLURM_COM_FLAG_ASN;
 	*result = (uint32_t) int_tmp;
@@ -133,7 +126,7 @@ set_prefix(json_t *object, bool is_assertion, struct slurm_prefix *result,
 	error = json_get_string(object, PREFIX, &str_prefix);
 	if (error && error == -ENOENT) {
 		if (is_assertion)
-			return pr_err("SLURM assertion prefix is required");
+			return pr_op_err("SLURM assertion prefix is required");
 		else
 			return 0; /* Optional for filters */
 	} else if (error)
@@ -141,7 +134,7 @@ set_prefix(json_t *object, bool is_assertion, struct slurm_prefix *result,
 
 	clone = strdup(str_prefix);
 	if (clone == NULL)
-		return -pr_errno(errno, "Couldn't allocate string to parse prefix");
+		return -pr_op_errno(errno, "Couldn't allocate string to parse prefix");
 
 	token = strtok(clone, "/");
 	isv4 = strchr(token, ':') == NULL;
@@ -200,11 +193,11 @@ set_max_prefix_length(json_t *object, bool is_assertion, uint8_t addr_fam,
 
 	/* Unsupported by filters */
 	if (!is_assertion)
-		return pr_err("Prefix filter can't have a max prefix length");
+		return pr_op_err("Prefix filter can't have a max prefix length");
 
 	/* An underflow or overflow will be considered here */
 	if (int_tmp <= 0 || (addr_fam == AF_INET ? 32 : 128) < int_tmp)
-		return pr_err("Max prefix length (%lld) is out of range [1 - %d].",
+		return pr_op_err("Max prefix length (%lld) is out of range [1 - %d].",
 		    int_tmp, (addr_fam == AF_INET ? 32 : 128));
 
 	*flag = *flag | SLURM_PFX_FLAG_MAX_LENGTH;
@@ -214,7 +207,7 @@ set_max_prefix_length(json_t *object, bool is_assertion, uint8_t addr_fam,
 
 }
 
-int
+static int
 validate_base64url_encoded(const char *encoded)
 {
 	/*
@@ -223,7 +216,7 @@ validate_base64url_encoded(const char *encoded)
 	 * without trailing '=' (Section 5 of [RFC4648])"
 	 */
 	if (strrchr(encoded, '=') != NULL)
-		return pr_err("The base64 encoded value has trailing '='");
+		return pr_op_err("The base64 encoded value has trailing '='");
 
 	/*
 	 * IMHO there's an error at RFC 8416 regarding the use of base64
@@ -253,7 +246,7 @@ set_ski(json_t *object, bool is_assertion, struct slurm_bgpsec *result,
 	error = json_get_string(object, SKI, &str_encoded);
 	if (error && error == -ENOENT) {
 		if (is_assertion)
-			return pr_err("SLURM assertion %s is required", SKI);
+			return pr_op_err("SLURM assertion %s is required", SKI);
 		else
 			return 0; /* Optional for filters */
 	} else if (error)
@@ -270,7 +263,7 @@ set_ski(json_t *object, bool is_assertion, struct slurm_bgpsec *result,
 	/* Validate that's at least 20 octects long */
 	if (ski_len != RK_SKI_LEN) {
 		free(result->ski);
-		return pr_err("The decoded SKI must be 20 octets long");
+		return pr_op_err("The decoded SKI must be 20 octets long");
 	}
 
 	result->data_flag = result->data_flag | SLURM_BGPS_FLAG_SKI;
@@ -295,12 +288,12 @@ validate_router_spki(unsigned char *data, size_t len)
 	tmp = data;
 	spki = d2i_X509_PUBKEY(NULL, &tmp, len);
 	if (spki == NULL)
-		return crypto_err("Not a valid router public key");
+		return op_crypto_err("Not a valid router public key");
 
 	ok = X509_PUBKEY_get0_param(&alg, NULL, NULL, &pa, spki);
 	if (!ok) {
 		X509_PUBKEY_free(spki);
-		return crypto_err("X509_PUBKEY_get0_param() returned %d", ok);
+		return op_crypto_err("X509_PUBKEY_get0_param() returned %d", ok);
 	}
 
 	error = validate_certificate_public_key_algorithm_bgpsec(pa);
@@ -323,14 +316,14 @@ set_router_pub_key(json_t *object, bool is_assertion,
 	/* Required by assertions */
 	if (error && is_assertion) {
 		if (error == -ENOENT)
-			return pr_err("SLURM assertion %s is required",
+			return pr_op_err("SLURM assertion %s is required",
 			    ROUTER_PUBLIC_KEY);
 		return error;
 	}
 
 	/* Unsupported by filters */
 	if (!is_assertion)
-		return pr_err("BGPsec filter can't have a router public key");
+		return pr_op_err("BGPsec filter can't have a router public key");
 
 	error = validate_base64url_encoded(str_encoded);
 	if (error)
@@ -339,7 +332,7 @@ set_router_pub_key(json_t *object, bool is_assertion,
 	error = base64url_decode(str_encoded, &result->router_public_key,
 	    &spk_len);
 	if (error)
-		return pr_err("'%s' couldn't be decoded", str_encoded);
+		return pr_op_err("'%s' couldn't be decoded", str_encoded);
 
 	/*
 	 * Validate that "is the full ASN.1 DER encoding of the
@@ -369,14 +362,14 @@ init_slurm_prefix(struct slurm_prefix *slurm_prefix)
 }
 
 static int
-load_single_prefix(json_t *object, bool is_assertion)
+load_single_prefix(json_t *object, struct db_slurm *db, bool is_assertion)
 {
 	struct slurm_prefix result;
 	size_t member_count;
 	int error;
 
 	if (!json_is_object(object))
-		return pr_err("Not a valid JSON object");
+		return pr_op_err("Not a valid JSON object");
 
 	init_slurm_prefix(&result);
 	member_count = 0;
@@ -402,19 +395,19 @@ load_single_prefix(json_t *object, bool is_assertion)
 
 	/* A single comment isn't valid */
 	if (result.data_flag == SLURM_COM_FLAG_COMMENT)
-		return pr_err("Single comments aren't valid");
+		return pr_op_err("Single comments aren't valid");
 
 	/* A filter must have ASN and/or prefix */
 	if (!is_assertion) {
 		if ((result.data_flag &
 		    (SLURM_COM_FLAG_ASN | SLURM_PFX_FLAG_PREFIX)) == 0)
-			return pr_err("Prefix filter must have an asn and/or prefix");
+			return pr_op_err("Prefix filter must have an asn and/or prefix");
 
 		/* Validate expected members */
 		if (!json_valid_members_count(object, member_count))
-			return pr_err("Prefix filter has unknown members (see RFC 8416 section 3.3.1)");
+			return pr_op_err("Prefix filter has unknown members (see RFC 8416 section 3.3.1)");
 
-		error = db_slurm_add_prefix_filter(db, &result, cur_ctx);
+		error = db_slurm_add_prefix_filter(db, &result);
 		if (error)
 			return error;
 
@@ -428,13 +421,13 @@ load_single_prefix(json_t *object, bool is_assertion)
 
 	if ((result.data_flag & SLURM_PFX_FLAG_MAX_LENGTH) > 0 &&
 	    result.vrp.prefix_length > result.vrp.max_prefix_length)
-		return pr_err("Prefix length is greater than max prefix length");
+		return pr_op_err("Prefix length is greater than max prefix length");
 
 	/* Validate expected members */
 	if (!json_valid_members_count(object, member_count))
-		return pr_err("Prefix assertion has unknown members (see RFC 8416 section 3.4.1)");
+		return pr_op_err("Prefix assertion has unknown members (see RFC 8416 section 3.4.1)");
 
-	error = db_slurm_add_prefix_assertion(db, &result, cur_ctx);
+	error = db_slurm_add_prefix_assertion(db, &result);
 	if (error)
 		return error;
 
@@ -442,26 +435,23 @@ load_single_prefix(json_t *object, bool is_assertion)
 }
 
 static int
-load_prefix_array(json_t *array, bool is_assertion)
+load_prefix_array(json_t *array, struct db_slurm *db, bool is_assertion)
 {
 	json_t *element;
 	int index, error;
 
 	json_array_foreach(array, index, element) {
-		error = load_single_prefix(element, is_assertion);
+		error = load_single_prefix(element, db, is_assertion);
 		if (!error)
 			continue;
 		if (error == -EEXIST)
-			pr_err(
-			    "The prefix %s element \"%s\", is duplicated or covered by another %s; SLURM loading will be stopped.%s",
+			pr_op_err(
+			    "The prefix %s element \"%s\", covers or is covered by another assertion/filter; SLURM loading will be stopped. %s",
 			    (is_assertion ? "assertion" : "filter"),
 			    json_dumps(element, 0),
-			    (is_assertion ? "assertion" : "filter"),
-			    (cur_ctx > 0
-			    ? " TIP: More than 1 SLURM files were found, check if the prefix is contained in multiple files (see RFC 8416 section 4.2)."
-			    : ""));
+			    "TIP: More than 1 SLURM files were found, check if the prefix is contained in multiple files (see RFC 8416 section 4.2).");
 		else
-			pr_err(
+			pr_op_err(
 			    "Error at prefix %s, element \"%s\", SLURM loading will be stopped",
 			    (is_assertion ? "assertions" : "filters"),
 			    json_dumps(element, 0));
@@ -482,14 +472,14 @@ init_slurm_bgpsec(struct slurm_bgpsec *slurm_bgpsec)
 }
 
 static int
-load_single_bgpsec(json_t *object, bool is_assertion)
+load_single_bgpsec(json_t *object, struct db_slurm *db, bool is_assertion)
 {
 	struct slurm_bgpsec result;
 	size_t member_count;
 	int error;
 
 	if (!json_is_object(object))
-		return pr_err("Not a valid JSON object");
+		return pr_op_err("Not a valid JSON object");
 
 	init_slurm_bgpsec(&result);
 	member_count = 0;
@@ -514,7 +504,7 @@ load_single_bgpsec(json_t *object, bool is_assertion)
 
 	/* A single comment isn't valid */
 	if (result.data_flag == SLURM_COM_FLAG_COMMENT) {
-		pr_err("Single comments aren't valid");
+		pr_op_err("Single comments aren't valid");
 		error = -EINVAL;
 		goto release_router_key;
 	}
@@ -523,19 +513,19 @@ load_single_bgpsec(json_t *object, bool is_assertion)
 	if (!is_assertion) {
 		if ((result.data_flag &
 		    (SLURM_COM_FLAG_ASN | SLURM_BGPS_FLAG_SKI)) == 0) {
-			pr_err("BGPsec filter must have an asn and/or SKI");
+			pr_op_err("BGPsec filter must have an asn and/or SKI");
 			error = -EINVAL;
 			goto release_router_key;
 		}
 
 		/* Validate expected members */
 		if (!json_valid_members_count(object, member_count)) {
-			pr_err("BGPsec filter has unknown members (see RFC 8416 section 3.3.2)");
+			pr_op_err("BGPsec filter has unknown members (see RFC 8416 section 3.3.2)");
 			error = -EINVAL;
 			goto release_router_key;
 		}
 
-		error = db_slurm_add_bgpsec_filter(db, &result, cur_ctx);
+		error = db_slurm_add_bgpsec_filter(db, &result);
 		if (error)
 			goto release_router_key;
 
@@ -544,12 +534,12 @@ load_single_bgpsec(json_t *object, bool is_assertion)
 
 	/* Validate expected members */
 	if (!json_valid_members_count(object, member_count)) {
-		pr_err("BGPsec assertion has unknown members (see RFC 8416 section 3.4.2)");
+		pr_op_err("BGPsec assertion has unknown members (see RFC 8416 section 3.4.2)");
 		error = -EINVAL;
 		goto release_router_key;
 	}
 
-	error = db_slurm_add_bgpsec_assertion(db, &result, cur_ctx);
+	error = db_slurm_add_bgpsec_assertion(db, &result);
 	if (error)
 		goto release_router_key;
 
@@ -563,26 +553,23 @@ release_ski:
 }
 
 static int
-load_bgpsec_array(json_t *array, bool is_assertion)
+load_bgpsec_array(json_t *array, struct db_slurm *db, bool is_assertion)
 {
 	json_t *element;
 	int index, error;
 
 	json_array_foreach(array, index, element) {
-		error = load_single_bgpsec(element, is_assertion);
+		error = load_single_bgpsec(element, db, is_assertion);
 		if (!error)
 			continue;
 		if (error == -EEXIST)
-			pr_err(
-			    "The bgpsec %s element \"%s\", is duplicated or covered by another %s; SLURM loading will be stopped.%s",
+			pr_op_err(
+			    "The ASN at bgpsec %s element \"%s\", is duplicated in another assertion/filter; SLURM loading will be stopped. %s",
 			    (is_assertion ? "assertion" : "filter"),
 			    json_dumps(element, 0),
-			    (is_assertion ? "assertion" : "filter"),
-			    (cur_ctx > 0
-			    ? " TIP: More than 1 SLURM files were found, check if the ASN is contained in multiple files (see RFC 8416 section 4.2)."
-			    : ""));
+			    "TIP: More than 1 SLURM files were found, check if the ASN is contained in multiple files (see RFC 8416 section 4.2).");
 		else
-			pr_err(
+			pr_op_err(
 			    "Error at bgpsec %s, element \"%s\", SLURM loading will be stopped",
 			    (is_assertion ? "assertions" : "filters"),
 			    json_dumps(element, 0));
@@ -603,18 +590,18 @@ load_version(json_t *root)
 	error = json_get_int(root, SLURM_VERSION, &version);
 	if (error)
 		return error == -ENOENT ?
-		    pr_err("SLURM member '"SLURM_VERSION"' is required.") :
+		    pr_op_err("SLURM member '"SLURM_VERSION"' is required.") :
 		    error;
 
 	/* Validate data */
 	if (version != 1)
-		return pr_err("'%s' must be 1", SLURM_VERSION);
+		return pr_op_err("'%s' must be 1", SLURM_VERSION);
 
 	return 0;
 }
 
 static int
-load_filters(json_t *root)
+load_filters(json_t *root, struct db_slurm *db)
 {
 	json_t *filters, *prefix, *bgpsec;
 	size_t expected_members;
@@ -631,17 +618,17 @@ load_filters(json_t *root)
 
 	expected_members = 2;
 	if (!json_valid_members_count(filters, expected_members))
-		return pr_err(
+		return pr_op_err(
 		    "SLURM '%s' must contain only %lu members (RFC 8416 section 3.2)",
 		    VALIDATION_OUTPUT_FILTERS,
 		    expected_members);
 
 	/* Arrays loaded, now iterate */
-	error = load_prefix_array(prefix, false);
+	error = load_prefix_array(prefix, db, false);
 	if (error)
 		return error;
 
-	error = load_bgpsec_array(bgpsec, false);
+	error = load_bgpsec_array(bgpsec, db, false);
 	if (error)
 		return error;
 
@@ -649,7 +636,7 @@ load_filters(json_t *root)
 }
 
 static int
-load_assertions(json_t *root)
+load_assertions(json_t *root, struct db_slurm *db)
 {
 	json_t *assertions, *prefix, *bgpsec;
 	size_t expected_members;
@@ -666,16 +653,16 @@ load_assertions(json_t *root)
 
 	expected_members = 2;
 	if (!json_valid_members_count(assertions, expected_members))
-		return pr_err(
+		return pr_op_err(
 		    "SLURM '%s' must contain only %lu members (RFC 8416 section 3.2)",
 		    LOCALLY_ADDED_ASSERTIONS,
 		    expected_members);
 
-	error = load_prefix_array(prefix, true);
+	error = load_prefix_array(prefix, db, true);
 	if (error)
 		return error;
 
-	error = load_bgpsec_array(bgpsec, true);
+	error = load_bgpsec_array(bgpsec, db, true);
 	if (error)
 		return error;
 
@@ -683,36 +670,37 @@ load_assertions(json_t *root)
 }
 
 static int
-handle_json(json_t *root, struct db_slurm *db_slurm, unsigned int *ctx)
+handle_json(json_t *root, struct db_slurm *db)
 {
 	size_t expected_members;
 	int error;
 
 	if (!json_is_object(root))
-		return pr_err("The root of the SLURM is not a JSON object.");
-
-	cur_ctx = *ctx;
-	db = db_slurm;
+		return pr_op_err("The root of the SLURM is not a JSON object.");
 
 	error = load_version(root);
 	if (error)
 		return error;
 
-	error = load_filters(root);
+	/* Start working on the cache */
+	db_slurm_start_cache(db);
+
+	error = load_filters(root, db);
 	if (error)
 		return error;
 
-	error = load_assertions(root);
+	error = load_assertions(root, db);
 	if (error)
 		return error;
 
 	expected_members = 3;
 	if (!json_valid_members_count(root, expected_members))
-		return pr_err(
+		return pr_op_err(
 		    "SLURM root must have only %lu members (RFC 8416 section 3.2)",
 		    expected_members);
 
-	(*ctx)++; /* Next time will be another context */
+	/* Persist cached data */
+	db_slurm_flush_cache(db);
 
 	return 0;
 }
