@@ -20,27 +20,25 @@ check_vrps_updates(void *param_void)
 	int error;
 
 	do {
+		sleep(config_get_validation_interval());
+
 		error = vrps_update(&changed);
 		if (error == -EINTR)
 			break; /* Process interrupted, terminate thread */
 
 		if (error) {
-			pr_err("Error code %d while trying to update the ROA database. Sleeping...",
-			    error);
-			goto sleep;
+			pr_op_err("Error while trying to update the ROA database. Sleeping...");
+			continue;
 		}
 
 		if (changed) {
 			error = notify_clients();
 			if (error)
-				pr_debug("Could not notify clients of the new VRPs. (Error code %d.) Sleeping...",
+				pr_op_debug("Couldn't notify clients of the new VRPs. (Error code %d.) Sleeping...",
 				    error);
 			else
-				pr_debug("Database updated successfully. Sleeping...");
+				pr_op_debug("Database updated successfully. Sleeping...");
 		}
-
-sleep:
-		sleep(config_get_validation_interval());
 	} while (true);
 
 	return NULL;
@@ -49,9 +47,16 @@ sleep:
 int
 updates_daemon_start(void)
 {
+	bool changed;
+	int error;
+
+	error = vrps_update(&changed);
+	if (error)
+		return pr_op_err("First validation wasn't successful.");
+
 	errno = pthread_create(&thread, NULL, check_vrps_updates, NULL);
 	if (errno)
-		return -pr_errno(errno,
+		return -pr_op_errno(errno,
 		    "Could not spawn the update daemon thread");
 
 	return 0;
