@@ -10,6 +10,9 @@
 static char addr_buffer1[INET6_ADDRSTRLEN];
 static char addr_buffer2[INET6_ADDRSTRLEN];
 
+static unsigned int http_priority = 60;
+static unsigned int rsync_priority = 50;
+
 char const *
 v4addr2str(struct in_addr const *addr)
 {
@@ -40,6 +43,12 @@ fnstack_peek(void)
 	return NULL;
 }
 
+void
+reqs_errors_log_summary(void)
+{
+	/* Nothing here */
+}
+
 char const *
 config_get_tal(void)
 {
@@ -79,19 +88,19 @@ config_get_rsync_enabled(void)
 unsigned int
 config_get_rsync_priority(void)
 {
-	return 50;
+	return rsync_priority;
 }
 
 bool
-config_get_rrdp_enabled(void)
+config_get_http_enabled(void)
 {
 	return false;
 }
 
 unsigned int
-config_get_rrdp_priority(void)
+config_get_http_priority(void)
 {
-	return 50;
+	return http_priority;
 }
 
 bool
@@ -149,18 +158,90 @@ config_get_output_bgpsec(void)
 	return NULL;
 }
 
+bool
+config_get_op_log_enabled(void)
+{
+	return true;
+}
+
+char const *
+config_get_op_log_tag(void)
+{
+	return NULL;
+}
+
+bool
+config_get_op_log_color_output(void)
+{
+	return false;
+}
+
+enum filename_format
+config_get_op_log_filename_format(void)
+{
+	return FNF_NAME;
+}
+
 uint8_t
-config_get_log_level(void)
+config_get_op_log_level(void)
 {
 	return 3; /* LOG_ERR */
 }
 
 enum log_output
-config_get_log_output(void)
+config_get_op_log_output(void)
 {
 	return CONSOLE;
 }
 
+uint32_t
+config_get_op_log_facility(void)
+{
+	return (3<<3); /* LOG_DAEMON */
+}
+
+bool
+config_get_val_log_enabled(void)
+{
+	return true;
+}
+
+char const *
+config_get_val_log_tag(void)
+{
+	return "VALIDATION";
+}
+
+bool
+config_get_val_log_color_output(void)
+{
+	return false;
+}
+
+enum filename_format
+config_get_val_log_filename_format(void)
+{
+	return FNF_NAME;
+}
+
+uint8_t
+config_get_val_log_level(void)
+{
+	return 3; /* LOG_ERR */
+}
+
+enum log_output
+config_get_val_log_output(void)
+{
+	return CONSOLE;
+}
+
+uint32_t
+config_get_val_log_facility(void)
+{
+	return (3<<3); /* LOG_DAEMON */
+}
+
 unsigned int
 config_get_asn1_decode_max_stack(void)
 {
@@ -208,3 +289,15 @@ config_get_http_ca_path(void)
 {
 	return NULL;
 }
+
+void
+config_set_rsync_priority(unsigned int value)
+{
+	rsync_priority = value;
+}
+
+void
+config_set_http_priority(unsigned int value)
+{
+	http_priority = value;
+}
